#========================================================================================
# package      DE_EPAGES::Database::API::DBIgalera
# base         DE_EPAGES::Database::API::DBImysql
# description  Galera specific implementation of DE_EPAGES::Database::API::DBI.
#========================================================================================
package DE_EPAGES::Database::API::DBIgalera;
use base qw (DE_EPAGES::Database::API::DBImysql);

use strict;

use DE_EPAGES::Database::API::Constants qw ( NUMBER_OF_RETRIES RETRY_WAITING_TIME );
use Time::HiRes qw ( usleep );

use constant DEADLOCK_ERR => 1213;  ## no critic qw(ProhibitMagicNumbers)
use constant DEADLOCK_STATE => 40001;  ## no critic qw(ProhibitMagicNumbers)

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Database::API::DBIgalera->new( $DataSource, $User, $Password, $CacheKey );
# example      DE_EPAGES::Database::API::DBIgalera->new( 'dbi:galera:storedb', 'root', 'geheim', 'DAL' );
#----------------------------------------------------------------------------------------
# description  Creates a database connection using the supplied connection parameters.
#----------------------------------------------------------------------------------------
# input        $DataSource | DBI connection string without user name/password | string
# input        $User | Database user or login name | string
# input        $Password | Database user password | string
# input        $CacheKey | key to identify the connection | string
# return       $dbi | database handle | DE_EPAGES::Database::API::DBImysql
#========================================================================================
sub new {
    my $class = shift;
    my ($DataSource, $User, $Password, $CacheKey) = @_;

    # Since the datasource name is used to load perl modules, that handle the interaction
    # with the database, we need to change this back to mysql.
    $DataSource =~ s/galera/mysql/;

    my $self = __PACKAGE__->SUPER::new($DataSource, $User, $Password, $CacheKey);

    return bless $self, $class;
}

#========================================================================================
# function     lockTables
# state        public
#----------------------------------------------------------------------------------------
# syntax       $dbi->lockTables( $ahTables, $cCode );
# example      $dbi->lockTables( [
#                   { 'table' => 'test', 'mode' => 'WRITE'},
#                   { 'table' => 'test2', 'mode' => 'WRITE'},
#               ], sub {
#                   $dbi->do('DELETE FROM test2 WHERE id = ?', $id);
#                   $dbi->do('DELETE FROM test WHERE id = ?', $id);
#               });
#----------------------------------------------------------------------------------------
# description  Since galera does not support the lock tables operation, we try to emulate
#               this with supported functionality. Instead of locking a table, we execute
#               the code inside of an transaction with the isolation level
#               'repeatable read'. Since 'repeatable read' is the default isolation level
#               we do not need to set it manually.
#----------------------------------------------------------------------------------------
# input        $ahTables | tables and locking modes. Hash keys are:
#                   <ul>
#                   <li>table - table name - string</li>
#                   <li>mode - locking mode (READ|WRITE) - string</li>
#                   </ul> | ref.array.hash
# input        $cCode | code block | code ref
#========================================================================================
sub lockTables {
    my $self = shift;
    my ( $ahTables, $cCode ) = @_;

    $self->transaction($cCode);

    return;
}

#========================================================================================
# function     transaction
# state        public
#----------------------------------------------------------------------------------------
# syntax       $dbi->transaction( $cCode, $cRollback );
# example      $dbi->transaction( sub {
#                   $dbi->do( "DELETE FROM objects WHERE classid = 25" );
#                   $dbi->do( "DELETE FROM classes WHERE classid = 25" );
#               } );
#----------------------------------------------------------------------------------------
# description  Executes a code block in a transaction.
#----------------------------------------------------------------------------------------
# input        $cCode | code block | code reference
# input        $cRollback | roll back code block (optional) | code reference
#========================================================================================
sub transaction {
    my $self = shift;
    my ($cCode, $cRollback) = @_;

    my $TryCounter = 0;

    do {
        $TryCounter++;
        $self->SUPER::transaction($cCode, $cRollback);
    } while ( $self->_dbDeadlock() and $TryCounter < NUMBER_OF_RETRIES and usleep(RETRY_WAITING_TIME) );

    return;
}

sub _dbDeadlock {
    my $self = shift;

    my $dbh = $self->dbh;

    if ( $dbh->err == DEADLOCK_ERR and $dbh->state == DEADLOCK_STATE ) {
        return 1;
    } else {
        return 0;
    }
}

1;
