#========================================================================================
# package      DE_EPAGES::Ebay::API::CSV::EbayAuctionExportImport;
# state        public
#----------------------------------------------------------------------------------------
# description  This module implements the high level interface for import and export of
#               ebay auction data to CVS files.
#========================================================================================
package DE_EPAGES::Ebay::API::CSV::EbayAuctionExportImport;
use base DE_EPAGES::ShopCSVExportImport::API::CSV::ShopObjectExportImport;

use strict;
use DE_EPAGES::Ebay::API::Constants qw( @CSV_WARNING_CODES_EBAYAUCTION );
use DE_EPAGES::Ebay::API::CSV::EbayAuctionExportDriver;
use DE_EPAGES::Ebay::API::CSV::EbayAuctionImportHandler;
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadObject );
use DE_EPAGES::Core::API::Warning qw(Warning);

#========================================================================================
# function     warningCodes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aWarningCodes = $Driver->warningCodes();
#----------------------------------------------------------------------------------------
# description  Returns the warning codes for csv export.
#----------------------------------------------------------------------------------------
# return       $aWarningCodes | warning codes | array.string
#========================================================================================
sub warningCodes {
    my $self = shift;
    return [ @{$self->SUPER::warningCodes}, @CSV_WARNING_CODES_EBAYAUCTION ];
}

#========================================================================================
# function     exportDriver
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Driver->exportDriver();
#----------------------------------------------------------------------------------------
# description  Return driver package.
#----------------------------------------------------------------------------------------
# return       $DriverPackage  | package to driver | string
#========================================================================================
sub exportDriver {
    my $self = shift;
    return 'DE_EPAGES::Ebay::API::CSV::EbayAuctionExportDriver';
}

#========================================================================================
# function     importDriver
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Driver->importDriver();
#----------------------------------------------------------------------------------------
# description  Return driver package.
#----------------------------------------------------------------------------------------
# return       $DriverPackage  | package to driver | string
#========================================================================================
sub importDriver {
    my $self = shift;
    return 'DE_EPAGES::Ebay::API::CSV::EbayAuctionImportHandler';
}

#========================================================================================
# function     getExportObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Driver->getExportObjects($Shop, $aObjects);
#----------------------------------------------------------------------------------------
# description  Returns list of objects to export.
#----------------------------------------------------------------------------------------
# input        $Shop           | alias of shop          | object
# input        $aObjects       | list of exportable objects | ref.array.integer
# return       $aExportObjects | edited list of exportable objects | ref.array
#========================================================================================
sub getExportObjects {
    my $self = shift;
    my ($Shop, $aObjects) = @_;


    my $aAuctionIDs;
    if (not defined $aObjects){
        my @Objects;
        push @Objects, @{$Shop->folder('EbayAuctions')->findChildren} if defined $Shop->child('EbayAuctions');
        my $ItemCount = scalar(@Objects);
        Warning('NO_EBAYAUCTIONS') unless $ItemCount;
        return \@Objects;
    } else{
        foreach my $AuctionID (@$aObjects){
            push @$aAuctionIDs, $AuctionID;
            my $Auction = LoadObject($AuctionID);
            push @$aAuctionIDs, $_->id foreach @{$Auction->get('SubAuctions')};
        }
        my $ItemCount = scalar(@$aAuctionIDs);
        Warning('NO_EBAYAUCTIONS') unless $ItemCount;
    }

    return $aAuctionIDs;
}

#========================================================================================
# function     getAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $aObjectattributes = $Driver->getAttributes($Shop);
#----------------------------------------------------------------------------------------
# description  Return list of exportable attributes.
#----------------------------------------------------------------------------------------
# return       $aAttributes | list of attributes | ref.array.object
#========================================================================================
sub getAttributes {
    my $self = shift;
    my $Class = LoadClassByAlias('EbayAuction');
    return [map { $Class->attribute($_) } qw (
        Alias
        ParentAuction
        Product
        ListingType
        IsReviseInventory
        EbayShopEnvironment
        Name
        SubTitle
        Description
        EbayUserTemplate

        EbayCategoryId

        Quantity
        StartPrice
        IsBuyItNowEnabled
        BuyItNowPrice
        IsGetItFast
        IsPrivate
        ScheduleType
        ScheduleTime

        Duration
        DispatchTimeMax
        ReturnsWithin
        ConditionID
        IsBestOfferEnabled
        BestOfferAutoAcceptPrice
        MinimumBestOfferPrice

        IsBoldTitle
        IsHighlight
        IsBordered
        HitCounter
        IsFeatured
        IsHomePageFeatured
        Gallery
        GalleryUrl

        PictureUrl
        PicVariationAttribute
        PicVariationType

        BusinessPolicyShippingID
        BusinessPolicyPaymentID
        BusinessPolicyReturn_PolicyID

    )];
}

1;
