#========================================================================================
# package      DE_EPAGES::Ebay::API::Object::EbayAuction
# state        public
# base         DE_EPAGES::Shop::API::Object::ShopObject
# description  object interface for EbayAuction.
#========================================================================================
package DE_EPAGES::Ebay::API::Object::EbayAuction;
use base DE_EPAGES::Shop::API::Object::ShopObject;

use strict;
use DE_EPAGES::Object::API::Factory qw (LoadClassByAlias);
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::Database::API::Connection                    qw (GetCurrentDBHandle);
use DE_EPAGES::Ebay::API::Tools qw ( ConvertFromEbayDate CheckEbayAuctionsFeature SetNewEbayAuctionsFeatureValue );
use DE_EPAGES::Ebay::API::Constants;
use DE_EPAGES::Ebay::API::Table::EbayAuctionAttribute qw ( DeleteEbayAuctionAttribute InsertEbayAuctionAttribute GetInfoEbayAuctionAttributesByAuctionID GetPKeyEbayAuctionAttributesByAuctionID );## no critic qw(ProhibitLongLines)
use DE_EPAGES::Core::API::PerlTools qw (isValid);
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use JSON qw ( from_json to_json );

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $EbayAuction->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'EbayAuction' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $EbayAuction->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $EbayAuction = DE_EPAGES::Ebay::API::Object::EbayAuction->insert($hInfo);
# example      my $Class = LoadClassByAlias('EbayAuction');
#               my $EbayAuction = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertEbayAuction' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $EbayAuction | new object | object
# hook         OBJ_InsertEbayAuction | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>AuctionID     - object id of Auction - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $self = $class->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertEbayAuction', {'Object' => $self, 'AuctionID' => $self->id });
    delete $hInfo->{$_} foreach qw ( Shop Product EbayShopEnvironment EbayUserTemplate ExternalAuctionId EbayUUID Name SubTitle EbayCategoryId ListingType Quantity StartPrice Duration ScheduleType ScheduleTime IsBuyItNowEnabled BuyItNowPrice IsPrivate HitCounter IsBoldTitle IsHighlight IsBordered IsFeatured IsHomePageFeatured Gallery PictureUrl GalleryUrl Status CurrentBid BidCount StartTime EndTime Description IsGetItFast SecondCategoryId IsNew AcceptsReturnPolicy DispatchTimeMax ReturnsWithin ConditionID AuctionTemplateID IsDefaultAuction UserTemplateOnly ParentAuction IsReviseInventory IsBestOfferEnabled BestOfferAutoAcceptPrice MinimumBestOfferPrice BusinessPolicyPaymentID BusinessPolicyShippingID BusinessPolicyReturn_PolicyID); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $EbayAuction->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteEbayAuction' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteEbayAuction | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>AuctionID     - object id of Auction - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteEbayAuction', {'Object' => $self, 'AuctionID' => $self->id });
    $_->set({ 'AuctionTemplateID' => undef }) foreach @{LoadClassByAlias('EbayAuction')->getObjects('ByAuctionTemplateID', [$self->id])};
    LoadClassByAlias('EbayTransaction')->deleteObjects('ByEbayAuctionID', [$self->id]);
    $_->set({}) foreach @{LoadClassByAlias('EbayTransaction')->getObjects('BySubProductAuctionID', [$self->id])};
    LoadClassByAlias('EbayAuction')->deleteObjects('ByParentAuctionID', [$self->id]);
    DeleteEbayAuctionAttribute($_) foreach @{GetPKeyEbayAuctionAttributesByAuctionID($self->id)};
    LoadClassByAlias('EbayAuctionPaymentMethod')->deleteObjects('ByEbayAuctionID', [$self->id]);
    LoadClassByAlias('EbayAuctionShippingLocation')->deleteObjects('ByEbayAuctionID', [$self->id]);
    LoadClassByAlias('EbayAuctionShippingMethod')->deleteObjects('ByEbayAuctionID', [$self->id]);
    $self->set({ 'AuctionTemplateID' => undef });
    $self->set({ 'ParentAuction' => undef });
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $EbayAuction->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateEbayAuction' and 'OBJ_AfterUpdateEbayAuction'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateEbayAuction | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>AuctionID     - object id of Auction - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateEbayAuction | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>AuctionID     - object id of Auction - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateEbayAuction', {'Object'=>$self, 'Values'=> $hValues, 'AuctionID' => $self->id, 'LanguageID' => $LanguageID});
    for my $attribute (qw (StartPrice BuyItNowPrice)) {
        if ($hValues->{$attribute}) {
            my $EbayShopEnvironment = $self->get('EbayShopEnvironment');
            return unless $EbayShopEnvironment;
            my $env = $EbayShopEnvironment->get('EbayEnvironment');
            return unless $env;
            $hValues->{$attribute} = $self->getSite()->formatter($env->get('CurrencyID'))->round( $hValues->{$attribute} );
        }
    }
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateEbayAuction',  {'Object'=>$self, 'Values'=> $hValues, 'AuctionID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     clone
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Clone = $EbayAuction->clone;
#----------------------------------------------------------------------------------------
# description  Clones the Auction and Payment- ,ShippingMethods and ShippingLocations.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
#----------------------------------------------------------------------------------------
# return       $RemoteEbay | perl package for communication between epages and ebay | object
#========================================================================================
sub clone {
    my $self = shift;

    my ($hParams, $hBusinessPolicySellerProfiles) = @_;

    my $clone = $self->SUPER::clone($hParams);

    # set the new UUID because we can not upload 2 auctions with the same EbayUUID...
    my $EbayUUID = $clone->get('GUID');
    $EbayUUID =~ s/-//g;
    $clone->set({'EbayUUID' => $EbayUUID});

    # clone shipping, shippinglocation, payment...
    # fix for EPG-12279: reverse list if we have old auctions with position = 0 entries
    my @shippingList = @{$self->folder('AuctionShippingMethods')->children};
    my $maxPosition = 0;
    foreach my $ship (@shippingList){
        my $nextPosition = $ship->get('Position');
        $maxPosition = ($nextPosition > $maxPosition ? $nextPosition : $nextPosition);
    }

    my $Position = 0;
    foreach my $ship (($maxPosition == 0 ? reverse @shippingList : @shippingList)) {
        $Position += 10; ## no critic qw(ProhibitMagicNumbers)
        $clone->folder('AuctionShippingMethods')->insertChild('EbayAuctionShippingMethod', {
            'Alias'                 => $clone->folder('AuctionShippingMethods')->nextAlias('EbayAuctionShippingMethod%s'),
            'EbayShippingMethod'    => $ship->get('EbayShippingMethod'),
            'EbayAuction'           => $clone,
            'Price'                 => $ship->get('Price'),
            'AdditionalPrice'       => $ship->get('AdditionalPrice'),
            'Position'              => $Position
        });
    }

    $Position = 0;
    foreach my $shiploc (@{$self->folder('AuctionShippingLocations')->children}) {
        $clone->folder('AuctionShippingLocations')->insertChild('EbayAuctionShippingLocation', {
            'Alias'                 => $clone->folder('AuctionShippingLocations')->nextAlias('EbayAuctionShippingLocation%s'),
            'EbayShippingMethod'    => $shiploc->get('EbayShippingMethod'),
            'EbayShippingLocation'  => $shiploc->get('EbayShippingLocation'),
            'EbayAuction'           => $clone,
            'Price'                 => $shiploc->get('Price'),
            'AdditionalPrice'       => $shiploc->get('AdditionalPrice'),
            'Position'              => $Position,
        });
        $Position += 10; ## no critic qw(ProhibitMagicNumbers)
    }

    foreach my $payment (@{$self->folder('AuctionPaymentMethods')->children}) {
        $clone->folder('AuctionPaymentMethods')->insertChild('EbayAuctionPaymentMethod', {
            'Alias'                 => $clone->folder('AuctionPaymentMethods')->nextAlias('EbayAuctionPaymentMethod%s'),
            'EbayAuction'           => $clone,
            'EbayPaymentMethod'     => $payment->get('EbayPaymentMethod'),
            'CODCost'               => $payment->get('CODCost'),
        });
    }

    #clone the auctionAttributes
    my $ahEbayAuctionAttributes = GetInfoEbayAuctionAttributesByAuctionID($self->id);

    foreach my $hAttributeKey (@$ahEbayAuctionAttributes) {
        $hAttributeKey->{'AuctionID'} = $clone->id;
        InsertEbayAuctionAttribute($hAttributeKey);
    }

    # clone the subAuction for variation products
    if( $clone->get('Product')->get('IsMaster') ){
        foreach my $SubAuction ( @{$self->get('SubAuctions')} ) {
            $SubAuction->clone({
                'Alias'             => $clone->parent->nextAlias('EbayAuction%s'),
                'Status'            => AUCTION_EDITABLE,
                'ExternalAuctionId' => undef,
                'ScheduleType'      => 0,
                'ScheduleTime'      => undef,
                'CurrentBid'        => 0,
                'BidCount'          => 0,
                'StartTime'         => GetCurrentDBHandle()->currentDateTime,
                'EndTime'           => undef,
                'ParentAuction'     => $clone,
            }, $hBusinessPolicySellerProfiles);
       }
    }

    $clone->_addBusinessPolicy($hBusinessPolicySellerProfiles);

    return $clone;
}

#========================================================================================
# function     _addBusinessPolicy
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_addBusinessPolicy($clone);
#----------------------------------------------------------------------------------------
# description  Adds the default values for the business policies to an auction if they are not set yet
#
#               Business policies were introduced by ebay in (2013) and should be used instead of setting
#               the shipping, payment and return policy in the shop.
#----------------------------------------------------------------------------------------
# input        $cloneAuction | the cloned auction | object
#----------------------------------------------------------------------------------------
# return       $cloneAuction | the cloned auction | object
#========================================================================================
sub _addBusinessPolicy {
    my $auction = shift;
    my ($hSellerProfiles) = @_;

    my $EbayShopEnvironment = $auction->get('EbayShopEnvironment');

    my $hbusinessIDs
        = $auction->get(['BusinessPolicyPaymentID','BusinessPolicyShippingID','BusinessPolicyReturn_PolicyID']);

    return $auction if
        $auction->allBusinessValuesAreSet($hbusinessIDs);

    return $auction
        unless defined $EbayShopEnvironment;

    $hSellerProfiles = $EbayShopEnvironment->getSellerProfiles($hSellerProfiles);

    return $auction
        unless $EbayShopEnvironment->isBusinessPolicyActivated($hSellerProfiles);

    $auction->_setProfile($hbusinessIDs,$hSellerProfiles,$EbayShopEnvironment);

    return $auction;
}

#========================================================================================
# function     setProfile
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_setProfile($hbusinessIDs,$hSellerProfiles,$EbayShopEnvironment);
#----------------------------------------------------------------------------------------
# description  Adds the default values for the business policies to an auction if they are not set yet
#----------------------------------------------------------------------------------------
# input        $auction | the cloned auction | object
#----------------------------------------------------------------------------------------
# return       $auction | the cloned auction | object
#========================================================================================
sub _setProfile {
    my $auction = shift;
    my ($abusinessIDs,$hSellerProfiles,$EbayShopEnvironment) = @_;

    my $hDefaultProfile;

    foreach my $SellerProfileType ('Payment','Shipping','Return_Policy') {

        my $businessIDName = 'BusinessPolicy'.$SellerProfileType.'ID';

        unless ($abusinessIDs->{$businessIDName} ) {
            $hDefaultProfile = $EbayShopEnvironment->filterSellerProfilesByDefault($hSellerProfiles, $SellerProfileType);
            $auction->set({"$businessIDName" => $hDefaultProfile->{'ProfileID'}}) if defined $hDefaultProfile;
        }
    }

    return $auction;
}

#========================================================================================
# function     allBusinessValuesAreSet
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->allBusinessValuesAreSet(
#                           $businessPolicyPaymentID, $businessPolicyShippingID , $businessPolicyReturn_PolicyID);
#----------------------------------------------------------------------------------------
# description  Returns true if all three business policies are set with a value
#----------------------------------------------------------------------------------------
# input        $businessPolicyPaymentID | value of the "payment" business policy | string
# input        $businessPolicyShippingID | value of the "shipping" business policy | string
# input        $businessPolicyReturn_PolicyID | value of the "return" business policy | string
#----------------------------------------------------------------------------------------
# return       $Result | returns 1 if all three business policies are set with a value | booelan
#========================================================================================
sub allBusinessValuesAreSet {
    my $self = shift;
    my ($hbusinessIDs) = @_;

    return 1 if isValid($hbusinessIDs->{'BusinessPolicyPaymentID'}) &&
                isValid($hbusinessIDs->{'BusinessPolicyShippingID'}) &&
                isValid($hbusinessIDs->{'BusinessPolicyReturn_PolicyID'});
    return 0;
}

#========================================================================================
# function     remoteEbay
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RemoteEbay = $EbayAuction->remoteEbay;
#----------------------------------------------------------------------------------------
# description  Returns the <package DE_EPAGES::Ebay::API::Object::RemoteEbay> object for
#               the ebay shop environment which is assigned this this EbayAuction.
#----------------------------------------------------------------------------------------
# return       $RemoteEbay | perl package for communication between epages and ebay | object
#========================================================================================
sub remoteEbay {
    my $self = shift;

    return $self->get('EbayShopEnvironment')->remoteEbay;
}

#========================================================================================
# function     verify
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hVerify = $EbayAuction->verify;
#----------------------------------------------------------------------------------------
# description  Verify auction data.
#----------------------------------------------------------------------------------------
# return       $hVerify |  return info, see <function DE_EPAGES::Ebay::API::Object::RemoteEbay::verifyAddItem> | ref.hash
#========================================================================================
sub verify {
    my $self = shift;

    # check feature if shop allows an additional auction, Error('EbayFeatureLimitExceeded') if not
    CheckEbayAuctionsFeature( $self->getSite() );

    # check auction
    if($self->get('Product')->get('IsMaster') == 1){
        return $self->remoteEbay->verifyAddFixedPriceItem($self);
    } else {
        return $self->remoteEbay->verifyAddItem($self);
    }
}

#========================================================================================
# function     upload
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hUpload = $EbayAuction->upload;
#----------------------------------------------------------------------------------------
# description  upload auction to ebay platform and set the state on 'running'.
#----------------------------------------------------------------------------------------
# return       $hUpload | return info, see <function DE_EPAGES::Ebay::API::Object::RemoteEbay::addItem> | ref.hash
#========================================================================================
sub upload {
    my $self = shift;

    # check feature if shop allows an additional auction
    my $Shop = $self->getSite();
    my $newValue = CheckEbayAuctionsFeature( $Shop );

    my $result;
    # upload auction
    if($self->get('Product')->get('IsMaster') == 1){
        $result = $self->remoteEbay->addFixedPriceItem($self);
    } else {
        $result = $self->remoteEbay->addItem($self);
    }


    $self->set({
        'StartTime' => ConvertFromEbayDate($result->{'StartTime'}),
        'EndTime'   => ConvertFromEbayDate($result->{'EndTime'}),
        'Status'    => AUCTION_RUNNING,
        'ExternalAuctionId' => $result->{'ItemID'},
    });

    # set feature
    SetNewEbayAuctionsFeatureValue($Shop, $newValue);

    my $EbayStockDecreaseType = $self->getSite()->get('EbayStockDecreaseType');
    if (defined $EbayStockDecreaseType and $EbayStockDecreaseType eq 'OnUpload') {
        if($self->get('Product')->get('IsMaster')){
            foreach my $SubAuction ( @{$self->get('SubAuctions')} ) {
                my $product = $SubAuction->get('Product');
                $product->substractBundleFromStock($SubAuction->get('Quantity')) if defined $product;
            }
        } else {
            my $product = $self->get('Product');
            $product->substractBundleFromStock($self->get('Quantity')) if defined $product;
        }
    }
    return $result;
}

sub allBidders {
    my $self = shift;

    return $self->remoteEbay->getAllBidders($self->get('ExternalAuctionId'));
}

#========================================================================================
# function     end
# state        public
#----------------------------------------------------------------------------------------
# syntax       $EbayAuction->end;
#----------------------------------------------------------------------------------------
# description  end ebay auction and set state to ended.
#----------------------------------------------------------------------------------------
# input        $EndingReason | ending reason of item | string (optional)
#========================================================================================
sub end {
    my $self = shift;
    my ($EndingReason) = @_;

    $self->remoteEbay->endItem($self->get('ExternalAuctionId'), $EndingReason);
    $self->set({
        'EndTime'   => GetCurrentDBHandle()->currentDateTime,
        'Status'    => AUCTION_ENDED,
    });
}

#========================================================================================
# function     relist
# state        public
#----------------------------------------------------------------------------------------
# syntax       $EbayAuction->relist;
#----------------------------------------------------------------------------------------
# description  relist auction to ebay platform and set the state on 'running'.
#----------------------------------------------------------------------------------------
# return       $hUpload | return info, see <function DE_EPAGES::Ebay::API::Object::RemoteEbay::relistItem> | ref.hash
#========================================================================================
sub relist {
    my $self = shift;

    my $result;
    if ($self->get('ListingType') eq 'Dutch') {
        $result = $self->remoteEbay->relistItem($self);
    } else {
        $result = $self->remoteEbay->relistFixedPriceItem($self);
    }

    $self->set({
        'StartTime' => ConvertFromEbayDate($result->{'StartTime'}),
        'EndTime'   => ConvertFromEbayDate($result->{'EndTime'}),
        'Status'    => AUCTION_RUNNING,
        'ExternalAuctionId' => $result->{'ItemID'},
    });

    return $result;
}

#========================================================================================
# function     getItemSpecifics
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $aItemSpecifics = $EbayAuction->getItemSpecifics();
#----------------------------------------------------------------------------------------
# description  deserializes the ItemSpecifics field from the auction
#----------------------------------------------------------------------------------------
# return       $aItemSpecifics | item specifics array. used keys in hash: <ul>
#                 <li>Name - attribute name - string</li>
#                 <li>Value - attribute value - string</li>
#               </ul> | ref.array.hash
#========================================================================================
sub getItemSpecifics {
    my $self = shift;
    my $JSonText = $self->get('ItemSpecifics');
    return [] unless isValid($JSonText);
    my $aItemSpecifics;
    eval {
        $aItemSpecifics = from_json ($JSonText);
    };
    if (ExistsError()) {
        my $Error = GetError();
        Warning($Error);
    }
    return $aItemSpecifics;
}

#========================================================================================
# function     setItemSpecifics
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $aItemSpecifics = $EbayAuction->setItemSpecifics($aItemSpecifics);
#----------------------------------------------------------------------------------------
# description  deserializes the ItemSpecifics field from the auction
#----------------------------------------------------------------------------------------
# input       $aItemSpecifics | item specifics array. used keys in hash: <ul>
#                 <li>Name - attribute name - string</li>
#                 <li>Value - attribute value - string</li>
#               </ul> | ref.array.hash
#========================================================================================
sub setItemSpecifics {
    my $self = shift;
    my ($aItemSpecifics) = @_;
    my $JSonText = to_json ($aItemSpecifics);
    $self->set({'ItemSpecifics' => $JSonText});
    return;
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::Ebay::API::Object::EbayAuction->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    my $Shop = $Parent->getSite();
    my $HitCounter = $Shop->get('IsUnityRegistered') ? 'HiddenStyle' : 'NoHitCounter';
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'Shop' => $Parent->getSite(),
        'ScheduleType' => 0,
        'IsBuyItNowEnabled' => 0,
        'IsPrivate' => 0,
        'IsBoldTitle' => 0,
        'IsHighlight' => 0,
        'IsBordered' => 0,
        'IsFeatured' => 0,
        'IsHomePageFeatured' => 0,
        'Gallery' => 'None',
        'HitCounter' => $HitCounter,
        'ListingType' => 'Chinese',
        'StartTime' => GetCurrentDBHandle()->currentDateTime,
        'IsGetItFast' => 0,
        'IsNew' => 0,
        'AcceptsReturnPolicy' => 1,
        'IsDefaultAuction' => 0,
        'UserTemplateOnly' => 0,
        'IsReviseInventory' => 0,
        'IsBestOfferEnabled' => 0,
    };
}

sub getSellerProfile {
    my $self = shift;
    my ($hhBusinessPolicySellerProfiles) = @_;

    return $hhBusinessPolicySellerProfiles->{$self->get('EbayShopEnvironment')->alias};
}
1;
