#========================================================================================
# package      DE_EPAGES::Ebay::API::TLE::EbayHandler
# state        public
#----------------------------------------------------------------------------------------
# description  tle handler for ebay
#========================================================================================
package DE_EPAGES::Ebay::API::TLE::EbayHandler;
use base DE_EPAGES::TLE::API::LoopHandler;

use strict;
use DE_EPAGES::Core::API::Log       qw ( GetLog  LogError) ;
use DE_EPAGES::Core::API::Error     qw ( ExistsError GetError );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadObject );
use DE_EPAGES::WebInterface::API::ApplicationServer qw ( GetAppServer );

#========================================================================================
# function     RegisterHandlerProc
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RegisterHandlerProc( $hParams );
#----------------------------------------------------------------------------------------
# description  this function registers this attribute handle on the processor via hook
#               function on hook TLEProcessorRegistration
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments keys :
#               <ul>
#                   <li>Processor - tle processor - object
#               </ul> | ref.hash
#========================================================================================
sub RegisterHandlerProc {
    my ($Params) = @_;
    GetLog->debug( 'EbayHandler.RegisterHandlerProc' );

    __PACKAGE__->new()->register( $Params->{'Processor'} );
    return;
}

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler = new DE_EPAGES::Object::API::TLE::EbayHandler;
# example      my $PrefixHandler  = new DE_EPAGES::Object::API::TLE::EbayHandler;
#----------------------------------------------------------------------------------------
# description  create a new handler
#----------------------------------------------------------------------------------------
# return       $Handler | blessed object | object
#========================================================================================
sub new { bless {}, shift; }

#========================================================================================
# function     register
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->register( $Processor );
#----------------------------------------------------------------------------------------
# description  this function register this handle on the processor
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
#========================================================================================
sub register {
    my $self = shift;
    my ($Processor) = @_;
    GetLog->debug( 'EbayHandler.register' );

    $Processor->registerHandler('FunctionHandler', $self, 'AuctionsByShopEnviromentID');
    $Processor->registerHandler('FunctionHandler', $self, 'TransactionsByOrderID');
    $Processor->registerHandler('FunctionHandler', $self, 'PercentageByCurrentBid');
    $Processor->registerHandler('FunctionHandler', $self, 'GetSellerProfiles');
    $Processor->registerHandler('FunctionHandler', $self, 'FilterSellerProfiles');
    $Processor->registerHandler('FunctionHandler', $self, 'IsBusinessPoliciesActivated');
    $Processor->registerHandler('FunctionHandler', $self, 'GetBusinessPolicies');

    return;
}

#========================================================================================
# function     AuctionsByShopEnvironmentID
# state        public
#----------------------------------------------------------------------------------------
# syntax       #AuctionsByShopEnvironmentID(#EbayShopEnvironmentID)
# example      #IF(#AuctionsByShopEnvironmentID(#EbayShopEnvironmentID))) {ShopEnvironmentContainsAuctions} #ENDIF
#----------------------------------------------------------------------------------------
# description  Returns the number auf auctions of this EbayShopEnvironment
#               Note: the shop default auction does not count
#----------------------------------------------------------------------------------------
# return       $AuctionCount
#========================================================================================
sub AuctionsByShopEnviromentID {
    my $self = shift;
    my ($Processor, $aParams, $cTemplate) = @_;
    my ($EbayShopEnvironmentID) = @$aParams;

    my $Shop = LoadObject($EbayShopEnvironmentID)->getSite;
    my $DefaultAuctionID = $Shop->get('EbayDefaultAuction')->id;
    my $aAuctions = LoadClassByAlias('EbayAuction')->findObjects('ByEbayShopEnvironmentID', [$EbayShopEnvironmentID]);
    my $Count = scalar grep { $_ != $DefaultAuctionID } @$aAuctions;
    return $Count;
}

#========================================================================================
# function     TransactionsByOrderID
# state        public
#----------------------------------------------------------------------------------------
# syntax       #TransactionsByOrderID(#OrderID)
# example      #IF(#TransactionsByOrderID(#OrderID))) {IsEbayOrder} #ENDIF
#----------------------------------------------------------------------------------------
# description  Returns the number auf Ebay Transactions found for this order.
#----------------------------------------------------------------------------------------
# return       #TransactionCount | number of eBay transactions | int
#========================================================================================
sub TransactionsByOrderID { ## no critic qw(RequireCommentForExportedFunctions)
    my $self = shift;
    my ($Processor, $aParams, $cTemplate) = @_;
    my ($CustomerOrderID) = @$aParams;
    return scalar @{ LoadClassByAlias('EbayTransaction')->findObjects('ByCustomerOrderID',[$CustomerOrderID]) };
}

#========================================================================================
# function     PercentageByCurrentBid
# state        public
#----------------------------------------------------------------------------------------
# syntax       #PercentageByCurrentBid(#ListPrice, #CurrentBid)
# example      #SET("PIXEL", #FUNCTION("PercentageByCurrentBid", #ListPrice, #CurrentBid)
#----------------------------------------------------------------------------------------
# description  Returns a diff (from -100% to +100%)
#               Used to set a pixel on a percentage scale to identify how good the current
#               bid is in relation to the regular product price.
#----------------------------------------------------------------------------------------
# return       #PercentageDiff | difference in percent | float
#========================================================================================
sub PercentageByCurrentBid { ## no critic qw(RequireCommentForExportedFunctions)
    my $self = shift;
    my ($Processor, $aParams, $cTemplate) = @_;
    my ($ProductPrice, $CurrentBid) = @$aParams;
    return 0 unless $ProductPrice;

    my $prozDiff = ($CurrentBid - $ProductPrice) / $ProductPrice;

    if ($prozDiff > 0) {
        $prozDiff = 0.1 if ($prozDiff > 0.1); ## no critic qw(ProhibitMagicNumbers)
    }
    else {
        $prozDiff = -0.1 if ($prozDiff < -0.1); ## no critic qw(ProhibitMagicNumbers)
    }
    $prozDiff = $prozDiff * 10; ## no critic qw(ProhibitMagicNumbers)
    return $prozDiff;
}

#========================================================================================
# function     GetSellerProfiles
# state        public
#----------------------------------------------------------------------------------------
# syntax       #GetSellerProfiles(#_)
#----------------------------------------------------------------------------------------
# description  Returns the ebay seller profile of an ebay shop environment
#----------------------------------------------------------------------------------------
# return       hSellerProfiles | hash with attribute values of the seller profile | ref.hash
#========================================================================================
sub GetSellerProfiles(){
    my $self = shift;
    my ($Processor, $aParams, $cTemplate) = @_;

    my $Servlet = GetAppServer()->currentServlet;

    my $hSellerProfiles = ();

    eval {
        my $auction = $Servlet->object;
        my $ShopEnv = $auction->get('EbayShopEnvironment');

        $hSellerProfiles  = $ShopEnv->getSellerProfiles();

    };
    if( ExistsError() ) {
        LogError( GetError() );
    }

    return $hSellerProfiles;

}

#========================================================================================
# function     FilterSellerProfiles
# state        public
#----------------------------------------------------------------------------------------
# syntax       #FilterSellerProfiles(#SellerProfiles, "Payment")
#----------------------------------------------------------------------------------------
# description  Returns only the seller profiles of the given profile type
#----------------------------------------------------------------------------------------
# return       $hFilteredSellerProfiles | hash with attribute values of the seller profile | ref.hash
#========================================================================================
sub FilterSellerProfiles(){
    my $self = shift;
    my ($Processor, $aParams, $cTemplate) = @_;
    my ($aSellerProfiles, $profileType) = @$aParams;

    my $Servlet = GetAppServer()->currentServlet;

    my $hFilteredSellerProfiles = ();

    eval{
        my $auction = $Servlet->object;
        my $ShopEnv = $auction->get('EbayShopEnvironment');

        $hFilteredSellerProfiles = $ShopEnv->filterSellerProfiles($aSellerProfiles, $profileType);
    };
    if( ExistsError() ) {
        LogError( GetError() );
    }

    return $hFilteredSellerProfiles;

}

#========================================================================================
# function     GetBusinessPolicies
# state        public
#----------------------------------------------------------------------------------------
# syntax       #GetBusinessPolicies(#_)
#----------------------------------------------------------------------------------------
# description  Returns an array with all business policies for a shop environment
#----------------------------------------------------------------------------------------
# return       $hArray | array with all business policies | ref.array
#========================================================================================
sub GetBusinessPolicies {
    my $self = shift;

    my $Servlet = GetAppServer()->currentServlet;

    my $aBusinessPolicies = ();

    eval {

        my $ServletObject = $Servlet->object;
        my $ShopEnv;
        if ($ServletObject->instanceOf('Shop')) {
            #The servlet object is the shop - for default auction overview
            $ShopEnv = $ServletObject->get('EbayDefaultAuction')->get('EbayShopEnvironment');
        } else {
            #The servlet object is an auction
            $ShopEnv = $ServletObject->get('EbayShopEnvironment');
        }

        $aBusinessPolicies = $ShopEnv->getBusinessPolicies();

    };
    if( ExistsError() ) {
        LogError( GetError() );
    }

    return $aBusinessPolicies;
}

#========================================================================================
# function     IsBusinessPoliciesActivated
# state        public
#----------------------------------------------------------------------------------------
# syntax       #IsBusinessPoliciesActivated(#SellerProfiles)
#----------------------------------------------------------------------------------------
# description  Returns 1 if the business policy feature is activated for the used ebay account
#----------------------------------------------------------------------------------------
# input        $aSellerProfiles | the whole result of the seller profile call | ref.array
#----------------------------------------------------------------------------------------
# return       isBusinessPoliciesActivated | returns 1 if business policy is activate | boolean
#========================================================================================
sub IsBusinessPoliciesActivated {
    my $self = shift;
    my ($Processor, $aParams, $cTemplate) = @_;
    my ($aSellerProfiles) = @$aParams;

    my $Servlet = GetAppServer()->currentServlet;

    my $isBusinessPoliciesActivated;

      eval {

        my $ServletObject = $Servlet->object;

        my $ShopEnv;
        if ($ServletObject->instanceOf('Shop')) {
           #The servlet object is the shop - for default auction overview
           $ShopEnv = $ServletObject->get('EbayDefaultAuction')->get('EbayShopEnvironment');
        } else {
           #The servlet object is an auction
           $ShopEnv = $ServletObject->get('EbayShopEnvironment');
        }

        $isBusinessPoliciesActivated = $ShopEnv->isBusinessPolicyActivated($aSellerProfiles);

    };
    if( ExistsError() ) {
        LogError( GetError() );
    }

    return $isBusinessPoliciesActivated;

}



1;
