#========================================================================================
# package      DE_EPAGES::Ebay::API::Tools
# state        public
#----------------------------------------------------------------------------------------
# description  little helper functions for ebay
#========================================================================================
package DE_EPAGES::Ebay::API::Tools;
use base Exporter;
use strict;

our @EXPORT_OK = qw (
    GetStoreRuName
    ConvertFromEbayDate
    GetRecentCategories
    GetOldestStartTime
    CheckEbayAuctionsFeature
    SetNewEbayAuctionsFeatureValue
    GetAuctionsSellerProfiles
);

use DateTime::Format::ISO8601;
use URI::Escape qw( uri_escape_utf8 );
use DE_EPAGES::Core::API::Error qw( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw(GetLog);
use DE_EPAGES::Core::API::PerlTools qw (GetLocalTimeZone);
use DE_EPAGES::Ebay::DAL::Ebay qw ( DAL_GetOldestStartTimeByEbayShopEnvironmentID );
use DE_EPAGES::Ebay::DAL::Categories qw (DAL_GetRecentCategories);
use DE_EPAGES::Ebay::API::Constants;
use DE_EPAGES::Shop::API::Feature qw( LoadFeatureByAlias );

#========================================================================================
# function     GetStoreRuName
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetStoreRuName( $StoreUrl );
#----------------------------------------------------------------------------------------
# description  Gets a global unique name for this store to identify at eBay.
#----------------------------------------------------------------------------------------
# input        $StoreUrl | systems web url  | string
# return       $ruName | ru name for ebay | string
#========================================================================================
sub GetStoreRuName {
    my ($StoreUrl) = @_;
    $StoreUrl =~ s/[^A-Za-z0-9]/_/g;
    $StoreUrl =~ s/^https___//;
    $StoreUrl =~ s/_+$//;

    my $ruName = uri_escape_utf8( EBAYSTORERUNAMEPART . $StoreUrl );

    $ruName = substr($ruName, 0, 64); ## no critic (ProhibitMagicNumbers)

    return $ruName;
}

#========================================================================================
# function     ConvertFromEbayDate
# state        public
#----------------------------------------------------------------------------------------
# syntax       ConvertFromEbayDate( $DateString );
#----------------------------------------------------------------------------------------
# description  Parses an eBay DateTime string and returns a DateTime object.
#               Converts the time to the local server time zone.
#----------------------------------------------------------------------------------------
# input        $DateString | datetime string in ISO8601 | string
# return       $dt | converted date | DateTime object
#========================================================================================
sub ConvertFromEbayDate {
    my ($DateString) = @_;

    $DateString =~ s/.\d{3}Z/Z/;  # 2004-09-17T03:20:47.123Z
    $DateString =~ s/\s+/T/;      # 2006-03-31 07:11:16

    GetLog()->debug('EbayTime (in UTC): '.$DateString);

    my $dt;
    eval {
        my $formatter = DateTime::Format::ISO8601->new();
        $dt = $formatter->parse_datetime($DateString);
    };
    if (ExistsError()) {
        my $Error = GetError();
        GetLog()->warn('Could not convert eBay date: '.$Error->message);
    }

    # convert from UTC to local time
    $dt->set_time_zone( GetLocalTimeZone());

    GetLog()->debug('DateTimeObject: '.$dt->ymd.' '.$dt->hms.' '.$dt->time_zone->name);
    return $dt;
}

#========================================================================================
# function     GetRecentCategories
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetRecentCategories( $Auction );
#----------------------------------------------------------------------------------------
# description  Builds a list of the recently used categories. Limited to the last 10
#               categories.
#----------------------------------------------------------------------------------------
# input        $Auction | auction | EbayAuction object
# input        $NumbersOfCategories | number | Number of Category depth
# return       $raCategories | tleHash for each category | ref.array.hash
#========================================================================================
sub GetRecentCategories {
    my ($Auction, $NumbersOfCategories) = @_;

    $NumbersOfCategories = 3 unless $NumbersOfCategories;

    my $EbayShopEnvironment = $Auction->get('EbayShopEnvironment');
    return [] unless defined $EbayShopEnvironment;

    my $Env = $EbayShopEnvironment->get('EbayEnvironment');
    my $Shop = $Auction->getSite();
    my $ahRecentCategories = DAL_GetRecentCategories($Shop->id,$Env->id);

    my $MAX_RECENT_CATEGORIES = 10; ## no critic (ProhibitMagicNumbers)

    if (scalar @$ahRecentCategories > $MAX_RECENT_CATEGORIES) {
        splice @$ahRecentCategories, $MAX_RECENT_CATEGORIES;
    }

    my $EbayCategoryTree = DE_EPAGES::Ebay::API::Object::EbayCategoryTree->new($Env);
    foreach my $hCategory (@$ahRecentCategories) {
        my $ResultCategory = $EbayCategoryTree->getInfoEbayCategoryByExternalCategoryId($hCategory->{'EbayCategoryID'});
        if (defined $ResultCategory) {
            $hCategory = $ResultCategory;
            $hCategory->{'Path'} = $EbayCategoryTree->getCategoryPath($hCategory->{'EbayCategoryID'}, $NumbersOfCategories );
        } else {
            $hCategory->{'Name'} = '';
            $hCategory->{'Path'} = [];
        }
    }
    return $ahRecentCategories;
}

#========================================================================================
# function     GetOldestStartTime
# state        public
#----------------------------------------------------------------------------------------
# syntax       $oldestStartTime = GetOldestStartTime( $ShopEnvironment );
#----------------------------------------------------------------------------------------
# description  Gets the StartTime of the oldest still running or ended (not finished) auctions.
#               The value will be in converted to UTC.
#----------------------------------------------------------------------------------------
# input        $ShopEnv | shop environemnt | EbayShopEnvironment object
# return       $oldestStartTime | date (undef if no running auction) | DateTime object
#========================================================================================
sub GetOldestStartTime {
    my ( $ShopEnv ) = @_;

    my $oldestStartTime = DAL_GetOldestStartTimeByEbayShopEnvironmentID ($ShopEnv->id);

    return undef unless defined $oldestStartTime;

    # server time
    $oldestStartTime->set_time_zone(GetLocalTimeZone());
    # convert to eBay UTC
    $oldestStartTime->set_time_zone('UTC');

    # substract 60 seconds for security reasons
    $oldestStartTime->add(seconds => -60);
    GetLog->debug('oldestStartTime:'.$oldestStartTime);

    return $oldestStartTime;
}

#========================================================================================
# function     CheckEbayAuctionsFeature
# state        public
#----------------------------------------------------------------------------------------
# syntax       CheckEbayAuctionsFeature( $Shop );
#----------------------------------------------------------------------------------------
# description  Checks if the shop has enough feature values of EbayAuctions left
#----------------------------------------------------------------------------------------
# input        $Shop | Shop | DE_EPAGES::Shop::API::Object::Shop
# return       $newValue | new feature count | integer
#========================================================================================
sub CheckEbayAuctionsFeature {
    my ($Shop) = @_;

    my $Feature = LoadFeatureByAlias('EbayAuctions');

    my $newValue = $Shop->featureCurrentValue($Feature) + 1;
    my $maxValue = $Shop->featureMaxValue($Feature);

    if ($newValue > $maxValue) {
        Error('EbayFeatureLimitExceeded', {
            'Feature'  => $Feature->alias,
            'MaxValue' => $maxValue,
            'NewValue' => $newValue,
            'Shop'     => $Shop
        });
    }
    return $newValue;
}

#========================================================================================
# function     GetAuctionsSellerProfiles
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetAuctionsSellerProfiles( $aAuctionList );
#----------------------------------------------------------------------------------------
# description  Creates a Hash of seller Profiles for a given auction list
#               This is an optimization for if there are many auctions with the same
#               sellerprofiles (should be mostly the case).
#
#               We avoid to do an ebay call for every auction which improves the performance.
#----------------------------------------------------------------------------------------
# input        $aAuctions | array of auctions | ref.array
# return       $hhBusinessPolicySellerProfiles | hash with seller profiles| ref.hash
#========================================================================================
sub GetAuctionsSellerProfiles {
    my ($aAuctions) = @_;

    my $EbayShopEnvironment;
    my $hhBusinessPolicySellerProfiles = {};

    foreach my $Auction (@$aAuctions) {
        $EbayShopEnvironment = $Auction->get('EbayShopEnvironment');
        $hhBusinessPolicySellerProfiles->{$EbayShopEnvironment->alias} = $EbayShopEnvironment->getSellerProfiles;
    }

    return $hhBusinessPolicySellerProfiles;
}

#========================================================================================
# function     SetNewEbayAuctionsFeatureValue
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetNewEbayAuctionsFeatureValue( $Shop, $newValue );
#----------------------------------------------------------------------------------------
# description  Sets EbayAuctions feature value for a shop.
#----------------------------------------------------------------------------------------
# input        $Shop | Shop | DE_EPAGES::Shop::API::Object::Shop
# input        $newValue | new feature value | integer
#========================================================================================
sub SetNewEbayAuctionsFeatureValue {
    my ($Shop, $newValue) = @_;

    $Shop->featureCurrentValue('EbayAuctions', $newValue);

    return;
}

1;
