#========================================================================================
# package      DE_EPAGES::PayPalExpress::API::Object::PaymentMethodPayPalExpress
# state        public
# base         DE_EPAGES::Order::API::Object::ContainerPaymentMethod
#----------------------------------------------------------------------------------------
# description  object interface for Payment.
#========================================================================================
package DE_EPAGES::PayPalExpress::API::Object::PaymentMethodPayPalExpress;
use base DE_EPAGES::Order::API::Object::ContainerPaymentMethod;

use strict;
use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::ShopMailType::API::ShopMailType qw( GetShopMailType );
use DE_EPAGES::PayPalExpress::API::IconUrlLocalizer;

#========================================================================================
# function     featureName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FeatureName = $PaymentMethodPayPalExpress->featureName;
#----------------------------------------------------------------------------------------
# description  Returns the feature name 'PayPalExpress'.
#----------------------------------------------------------------------------------------
# return       $FeatureName | feature name (undef means no feature count) | String
#========================================================================================
sub featureName () { return 'PayPalExpress'; }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PaymentMethodPayPalExpress = $Package->insert($hInfo);
# example      my $Class = LoadClassByAlias('PaymentMethodPayPalExpress');
#               my $PaymentMethodPayPalExpress = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  enables mail type PayPalExpressPaymentDeclined
#----------------------------------------------------------------------------------------
# input        $Package | class package | string
# input        $hInfo | attributes of new object | ref.hash
# return       $PaymentMethodPayPalExpress | new object | object
#========================================================================================
sub insert {
    my ($Package, $hInfo) = @_;

    # test if we can use PayPalExpress as paymentmethod
    my $Shop = $hInfo->{Shop} || $hInfo->{Parent}->getSite();

    #--- prevent creation of two payment method instances in the same shop
    my $aPaymentMethods = $Shop->get('PaymentMethods');
    foreach my $PaymentMethod (@$aPaymentMethods) {
        if ($PaymentMethod->instanceOf('PaymentMethodPayPalExpress')) {
            Error('NoSecondInstanceAllowed', {'Shop' => $Shop});
        }
    }

    #--- activate email event
    my $MailType  = GetShopMailType( $Shop, 'PayPalExpressPaymentDeclined');
    $MailType->set({ 'IsActive' => 1 }) if $MailType;

    my $self = $Package->SUPER::insert($hInfo);
    $self->_setDefaultNameForAllLanguages();

    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PaymentMethodPayPalExpress->delete;
#----------------------------------------------------------------------------------------
# description  deactivates Mail type PayPalExpressPaymentDeclined for the shop
#========================================================================================
sub delete {
    my $self = shift;

    #--- deactivate email event
    my $MailType  = GetShopMailType( $self->getSite(), 'PayPalExpressPaymentDeclined');
    $MailType->set({ 'IsActive' => 0 }) if $MailType;
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::PayPalExpress::API::Object::PaymentMethodPayPalExpress->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;

    # delete preset name so we can set in later for all languages.
    delete($hInfo->{'Name'});

    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'HidePayPalExpress'  => 0,
        'AllowGuestCheckout' => 0,
        'TransactionType' => 0, #no ESC (instand update) by default
    };
}

#========================================================================================
# function     usableAtStorefront
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Usable = $PaymentMethodPayPalExpress->usableAtStorefront($hVars);
#----------------------------------------------------------------------------------------
# description  Tests if a payment method with the given currency is visible in the storefront.
#               Uses function <function DE_EPAGES::Payment::API::Object::PaymentMethod::price>
#               to evaluate if payment methode has a price (price can be 0).
#----------------------------------------------------------------------------------------
# input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gross or net price    - int
#               </ul> | ref.hash
# return       $Usable | true if payment method is usable in the storefront | boolean
#========================================================================================
sub usableAtStorefront {
    my $self    = shift;
    my ($hVars) = @_;

    return $self->get('PaymentType')->get('IsActivated') &&
           defined($self->get('APIUsername'))            &&
           defined($self->get('APIPassword'))            &&
           defined($self->get('APISignatur'))
    ;
}

#----------------------------------------------------------------------------------------
# private
#----------------------------------------------------------------------------------------
sub _setDefaultNameForAllLanguages {
    my $self = shift;

    my $Shop = $self->get('Shop');
    my $hInstalledLanguages = $Shop->get('Languages');

    foreach my $Language (@{ $hInstalledLanguages }) {

        my $LanguageID = $Language->{'LanguageID'};
        my $LanguageDefaultName = 'PayPal';

        $self->set({'Name' => $LanguageDefaultName}, $LanguageID);
    }
}

#========================================================================================
# function     getAlternativeCheckoutButtonInfo
# state        public
#----------------------------------------------------------------------------------------
# syntax       getAlternativeCheckoutButtonInfo( $Basket );
#----------------------------------------------------------------------------------------
# description  return hash with information about the oneclick button for this paymentmethod.
#               returns undef if preconditions for displaying the button are not met.
#----------------------------------------------------------------------------------------
# input        $Basket | basket object | object
# return       info hash | refhash
#========================================================================================
sub getAlternativeCheckoutButtonInfo {
    my $self = shift;
    my ($Basket) = @_;

    if($self->canAddToBasket() and
       $Basket->get('IsMinOrderAmountReached') and
       not $self->get('HidePayPalExpress')) {
        return $self->_getInfoHash($Basket);
    }

    return;
}

sub _getInfoHash {
    my $self = shift;
    my ($Basket) = @_;

    my $IconLocalizer = $self->_createIconUrlLocalizer();
    my $IconUrl = $IconLocalizer->getPayPalExpressIconUrl($Basket, $self);

    my $hInfo = {
        'name' => 'paypalexpress',
        'image' => $IconUrl,
        'paymentMethodId' => $self->id(),
    };

    return $hInfo;
}

sub _createIconUrlLocalizer {
    my $self = shift;
    return DE_EPAGES::PayPalExpress::API::IconUrlLocalizer->new();
}

1;
