#========================================================================================
# package      DE_EPAGES::Payment::API::Object::PaymentGroup
# state        public
#----------------------------------------------------------------------------------------
# description  object interface for PaymentGroup.
#========================================================================================
package DE_EPAGES::Payment::API::Object::PaymentGroup;
use base DE_EPAGES::Object::API::Object::Object;
use strict;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PaymentGroup = DE_EPAGES::Payment::API::Object::PaymentGroup->insert($hInfo);
# example      my $Class = LoadClassByAlias('PaymentGroup');
#               my $PaymentGroup = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertPaymentGroup' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $PaymentGroup | new object | object
# hook         OBJ_InsertPaymentGroup | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PaymentGroupID - provider group id    - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $self = $class->SUPER::insert($hInfo);
    TriggerHook('OBJ_InsertPaymentGroup', {'Object' => $self, 'PaymentGroupID' => $self->id() });
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PaymentGroup->delete();
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeletePaymentGroup' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeletePaymentGroup | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PaymentGroupID - payment group id    - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeletePaymentGroup', {'Object' => $self, 'PaymentGroupID' => $self->id() });
    $self->SUPER::delete();
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PaymentGroup->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdatePaymentGroup' and 'OBJ_AfterUpdatePaymentGroup'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdatePaymentGroup | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PaymentGroupID - payment group id    - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdatePaymentGroup | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PaymentGroupID - payment group id    - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdatePaymentGroup', {'Object'=>$self, 'Values'=> $hValues, 'PaymentGroupID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdatePaymentGroup',  {'Object'=>$self, 'Values'=> $hValues, 'PaymentGroupID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}
#========================================================================================
# function     isUsable
# state        public
#----------------------------------------------------------------------------------------
# syntax       isUsable( $Shop );
#----------------------------------------------------------------------------------------
# description  If at least one payment type is usable, it will return 1
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# return       true if one payment type of this group isUsable | boolean
#========================================================================================
sub isUsable {
    my $self = shift;
    my ($Shop) = @_;

    my $IsUsable;
    eval {
        my $aPaymentTypes = $self->getPaymentTypes($Shop);
        if(scalar @{ $aPaymentTypes } > 0) {
            $IsUsable = 1;
        } else {
            $IsUsable = 0;
        }
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogError($Error);
        $IsUsable = 0;
    }
    return $IsUsable;
}
#========================================================================================
# function     getPaymentTypes
# state        public
#----------------------------------------------------------------------------------------
# syntax       getPaymentTypes( $Shop );
#----------------------------------------------------------------------------------------
# description  returns all paymentTypes of this paymentgroup which are usable for this shop.
#----------------------------------------------------------------------------------------
# input        $Shop | Shop object | object
# return       $ReturnVariable | array of paymenttypes | refarray
#========================================================================================
sub getPaymentTypes {
    my $self = shift;
    my ($Shop) = @_;

    my $aPaymentTypes = [];

    my $aAllPaymentTypes = $self->_LoadObjectByPath('/PaymentTypes')->children();
    foreach my $PaymentType ( @{$aAllPaymentTypes} ){
      next unless defined $PaymentType->get('PaymentGroup');
      if($PaymentType->get('PaymentGroup')->alias() eq $self->alias()) {
        if($PaymentType->isUsable($Shop)){
            push(@{$aPaymentTypes},$PaymentType);
        }
      }
    }

    return $aPaymentTypes;
}

#----------------------------------------------------------------------------------------

sub _LoadObjectByPath {
    my $self = shift;
    my ( $Path, $RootObjectID ) = @_;
    return LoadObjectByPath( $Path, $RootObjectID );
}
1;