#========================================================================================
# package      DE_EPAGES::Payment::API::Object::PaymentType
# state        public
# base         DE_EPAGES::Shop::API::Object::ShopObject
# description  object interface for PaymentType.
#========================================================================================
package DE_EPAGES::Payment::API::Object::PaymentType;
use base DE_EPAGES::Shop::API::Object::ShopObject;

use strict;
use DE_EPAGES::Core::API::PerlTools qw ( Compare GetCurrentDateTime isValid );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );
use DE_EPAGES::Payment::API::Shop qw ( GetPaymentMethodsFolder );
use DE_EPAGES::Shop::API::Feature qw ( ExistsFeatureByAlias );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $PaymentType->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'PaymentType' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $PaymentType->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PaymentType = DE_EPAGES::Payment::API::Object::PaymentType->insert($hInfo);
# example      my $Class = LoadClassByAlias('PaymentType');
#               my $PaymentType = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertPaymentType' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $PaymentType | new object | object
# hook         OBJ_InsertPaymentType | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PaymentTypeID - payment type object id - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $self = $class->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertPaymentType', {'Object' => $self, 'PaymentTypeID' => $self->id });
    delete $hInfo->{$_} foreach qw ( MethodClass LineItemClass IsActivated IsHirePurchaseSupported IsCustomerRequired CanPaymentOnBasket CanPaymentOnOrder CanTransactionAuth CanTransactionCapture CanTransactionTest CanRegions Logging DeprecationDate ); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PaymentType->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeletePaymentType' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeletePaymentType | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PaymentTypeID - payment type object id - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeletePaymentType', {'Object' => $self, 'PaymentTypeID' => $self->id });
    LoadClassByAlias('PaymentMethod')->deleteObjects('ByPaymentTypeID', [$self->id]);
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PaymentType->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdatePaymentType' and 'OBJ_AfterUpdatePaymentType'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdatePaymentType | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PaymentTypeID - payment type object id - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdatePaymentType | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PaymentTypeID - payment type object id - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdatePaymentType', {'Object'=>$self, 'Values'=> $hValues, 'PaymentTypeID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdatePaymentType',  {'Object'=>$self, 'Values'=> $hValues, 'PaymentTypeID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::Payment::API::Object::PaymentType->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'IsActivated' => 1,
        'IsHirePurchaseSupported' => 0,
        'IsCustomerRequired' => 0,
        'CanPaymentOnBasket' => 1,
        'CanPaymentOnOrder' => 0,
        'CanTransactionAuth' => 1,
        'CanTransactionCapture' => 0,
        'CanTransactionTest' => 0,
        'CanRegions' => 1,
        'Logging' => 0,
        'CanDoMultiStepCheckout' => 1,
        'CanDoClassicCheckout' => 1,
        'CanDoMobileCheckout' => 1,
        'CanBeDefaultPayment' => 1,
    };
}

sub importXML {
    my ($ObjectPackage, $Handler, $hElement, $Class, $hAttributes) = @_;
    return $ObjectPackage->SUPER::importXML($Handler, $hElement, $Class, $hAttributes) if $Handler->topID('IsReference');

    $hAttributes->{'MethodClass'} = LoadClassByAlias($hAttributes->{'MethodClass'}) if defined $hAttributes->{'MethodClass'};
    $hAttributes->{'LineItemClass'} = LoadClassByAlias($hAttributes->{'LineItemClass'}) if defined $hAttributes->{'LineItemClass'};
    return $ObjectPackage->SUPER::importXML($Handler, $hElement, $Class, $hAttributes);
}

#========================================================================================
# function     exportXMLAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hAttributes = $Object->exportXMLAttributes( $Driver, $hAllAttributes );
#----------------------------------------------------------------------------------------
# description  Returns plain attributes of object to XML export driver. This function
#               removes bit attributes with value 0.
#----------------------------------------------------------------------------------------
# input        $Driver | xml export driver with support function addObject | object
# input        $hAllAttributes | all attributes of object,
#                                 see $Object-><function exportableAttributes> | ref.hash
# return       $hAttributes | plain attributes | ref.hash.string
#========================================================================================
sub exportXMLAttributes {
    my $self = shift;
    my ($Driver, $hAttributes) = @_;
    $hAttributes = $self->SUPER::exportXMLAttributes($Driver, $hAttributes);

    my $MethodClass = $self->get('MethodClass');
    $hAttributes->{'MethodClass'} = $MethodClass->alias if defined $MethodClass;

    my $LineItemClass = $self->get('LineItemClass');
    $hAttributes->{'LineItemClass'} = $LineItemClass->alias if defined $LineItemClass;

    return $hAttributes;
}

#========================================================================================
# function     newPaymentMethod
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $PaymentMethod = $PaymentType->newPaymentMethod($Shop, $hInfo);
#----------------------------------------------------------------------------------------
# description  Creates a new payment method with the given parameters.
#----------------------------------------------------------------------------------------
# input        $Shop | shop | object
# input        $hInfo | hash with attribute value for the new payment method,
#                       for example: { 'Alias' => $NewAlias } | ref.hash
# return       $PaymentMethod | payment method | object
#========================================================================================
sub newPaymentMethod {
    my $self = shift;
    my ($Shop, $hInfo) = @_;
    $hInfo->{'Parent'} = GetPaymentMethodsFolder($Shop);
    $hInfo->{'PaymentType'} = $self;
    $hInfo->{'Alias'} = $hInfo->{'Parent'}->nextAlias('PaymentMethod%4d') unless defined $hInfo->{'Alias'} and $hInfo->{'Alias'} ne '';
    my $MethodClass = $self->get('MethodClass');
    $MethodClass = LoadClassByAlias('PaymentMethod') unless defined $MethodClass;
    return $MethodClass->insertObject($hInfo);
}

#========================================================================================
# function     isUsable
# state        public
#----------------------------------------------------------------------------------------
# syntax       isUsable( $Shop );
#----------------------------------------------------------------------------------------
# description  returns if this paymenttype can be used in this shop
#----------------------------------------------------------------------------------------
# input        $Shop | Shop object | object
# return       $ReturnVariable | true if this paymenttype can be used | boolean
#========================================================================================
sub isUsable {
    my $self = shift;
    my ($Shop) = @_;

    return 0 if $self->_isDeprecated();
    return 0 unless $self->_isFeatured($Shop);

    return 1;
}
#----------------------------------------------------------------------------------------
sub _isDeprecated {
    my $self = shift;

    my $Now = $self->_GetCurrentDateTime();
    my $DeprecationDate = $self->get('DeprecationDate');

    return 0 unless isValid($DeprecationDate);

    if( Compare('DateTime', $Now, $DeprecationDate) >= 0) {
        return 1;
    } else {
        return 0;
    }

}
#----------------------------------------------------------------------------------------
sub _isFeatured {
    my $self = shift;
    my ($Shop) = @_;

    #If Feature does not exist, there is no feature restriction for this paymenttype.
    my $FeatureName = $self->get('MethodClass')->get('FeatureName');
    return 1 unless defined $FeatureName;
    return 1 unless $self->_ExistsFeatureByAlias($FeatureName);

    # The featureMaxValue of a paymenttype is 1 or 0
    if($Shop->featureMaxValue($FeatureName) > 0) {
        return 1;
    } else {
        return 0;
    }
}
#========================================================================================
# function     canCreateAdditionalPaymentMethod
# state        public
#----------------------------------------------------------------------------------------
# syntax       canCreateAdditionalPaymentMethod( $Shop );
#----------------------------------------------------------------------------------------
# description  check if it is possible to create an additional payment method of this type.
#               By default this will be true. It have to be overidden if a payment type
#               support only one payment method.
#----------------------------------------------------------------------------------------
# input        $Shop | epages shop object | DE_EPAGES::Shop::API::Object::Shop
# return       always true | boolean
#========================================================================================
sub canCreateAdditionalPaymentMethod {
    my $self = shift;
    my ($Shop) = @_;
    if(not $self->get('CanAddMultiplePaymentMethods')) {
        my $aPaymentMethods = $Shop->get('PaymentMethods');
        foreach my $PaymentMethod (@$aPaymentMethods) {
            if ($PaymentMethod->instanceOf($self->get('MethodClass')->alias())) {
                return 0;
            }
        }
    }
    return 1;
}
#----------------------------------------------------------------------------------------
sub _GetCurrentDateTime {
    my $self = shift;
    return GetCurrentDateTime();
}
#----------------------------------------------------------------------------------------
sub _ExistsFeatureByAlias {
    my $self = shift;
    my ($Alias) = @_;
    return ExistsFeatureByAlias($Alias);
}
1;
