#========================================================================================
# package      DE_EPAGES::Payment::API::PaymentInstaller
# base         DE_EPAGES::Shipping::API::ShippingInstaller
# state        public
#----------------------------------------------------------------------------------------
# description  Install/uninstall payment logos, types, methods and logomapping.
#               Processes Database/XML/PaymentLogos*.xml files.
#               Processes Database/XML/PaymentTypes*.xml files.
#               Use functions <function DE_EPAGES::Object::API::ObjectInstaller::onPatchDeleteObjects>,
#               <function DE_EPAGES::Object::API::ObjectInstaller::onPatchImportObjects>
#               to patch payment logos, types, methods and logomapping.
#========================================================================================
package DE_EPAGES::Payment::API::PaymentInstaller;
use base DE_EPAGES::Shipping::API::ShippingInstaller;

use strict;
use DE_EPAGES::Core::API::File qw ( RunOnGlob );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::Object::API::XML::AttributeImportHandler qw ( ImportTranslation );
use DE_EPAGES::Object::API::XML::ObjectDeleteHandler qw ( DeleteObjects );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::XML::API::TestSchema qw ( TestSchema );

#========================================================================================
# function     install
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->install
#----------------------------------------------------------------------------------------
# description  Installs Database/XML/PaymentLogos*.xml on installation process.
#               Installs Database/XML/PaymentGroups*.xml on installation process.
#               Installs Database/XML/PaymentTypes*.xml on installation process.
#========================================================================================
sub install {
    my $self = shift;

    $self->SUPER::install;

    my $XMLDir = $self->xmlDirectory;
    RunOnGlob($XMLDir, "PaymentLogos*.xml", sub { ImportObjects( $_ ) });
    RunOnGlob($XMLDir, "PaymentGroups*.xml", sub { ImportObjects( $_ ) }); 
    RunOnGlob($XMLDir, "PaymentTypes*.xml", sub { ImportObjects( $_ ) });
    ImportTranslation( "$XMLDir/Translation.PaymentTypes" );
    ImportTranslation( "$XMLDir/Translation.Notrans.PaymentTypes" );
    ImportTranslation( "$XMLDir/Translation.PaymentLogos" );
}

#========================================================================================
# function     uninstall
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstall;
#----------------------------------------------------------------------------------------
# description  Uninstalls Database/XML/PaymentTypes*.xml on uninstallation process.
#               Uninstalls Database/XML/PaymentGroups*.xml on uninstallation process.
#               Uninstalls Database/XML/PaymentLogos*.xml on uninstallation process.
#========================================================================================
sub uninstall {
    my $self = shift;

    my $XMLDir = $self->xmlDirectory;
    RunOnGlob($XMLDir, "PaymentTypes*.xml", sub { DeleteObjects( $_ ) }, {reverse=>1, force=>$self->force});
    RunOnGlob($XMLDir, "PaymentGroups*.xml", sub { DeleteObjects( $_ ) }, {reverse=>1, force=>$self->force});
    RunOnGlob($XMLDir, "PaymentLogos*.xml", sub { DeleteObjects( $_ ) }, {reverse=>1, force=>$self->force});

    $self->SUPER::uninstall;
}

#========================================================================================
# function     test
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->test;
#----------------------------------------------------------------------------------------
# description  runs xml schema validation for Taxes
#========================================================================================
sub test {
    my $self = shift;
    $self->SUPER::test();
    my $XMLDir = $self->xmlDirectory;
    my $XSDDir = $ENV{'EPAGES_CARTRIDGES'}.'/DE_EPAGES/Payment/Data/XSD/';
    TestSchema($XMLDir, 'PaymentTypes*.xml', [ $XSDDir.'v6/PaymentTypes.xsd' ] );
}


#========================================================================================
# function     onPatchUpdatePaymentMethod
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->onPatchUpdatePaymentMethod($File, $hParams);
#----------------------------------------------------------------------------------------
# example      $Installer->onPatchImportObjects(
#                         '', {
#                           'PaymentType' => 'WorldPay',
#                           'DoPaymentOnBasket' => '0',
#                         });
#----------------------------------------------------------------------------------------
# description  Called by PatchImportHandler by method onPatchUpdatePaymentMethod
#               Updates all payment methods of the given payment type.
#----------------------------------------------------------------------------------------
# input        $File | not used | string
# input        $hParams | patch file parameters, keys are
#               <ul>
#                   <li>PaymentType - payment type alias - string</li>
#                   <li>DoPaymentOnBasket - DoPaymentOnBasket flag - boolean</li>
#               </ul> | ref.hash.*
#========================================================================================
sub onPatchUpdatePaymentMethod {
    my $self = shift;
    my ( $File, $hParams ) = @_;

    my $dbi = GetCurrentDBHandle();
    my $PaymentTypeAlias = $hParams->{'PaymentType'};
    my $DoPaymentOnBasket = $hParams->{'DoPaymentOnBasket'};
    my $sql = <<END_SQL;
    UPDATE paymentmethod
    SET dopaymentonbasket = ?
    FROM paymentmethod pm
    INNER JOIN paymenttype pt ON pt.paymenttypeid = pm.paymenttypeid
    INNER JOIN object pto ON pto.objectid = pt.paymenttypeid
    WHERE alias = ?
END_SQL

    $dbi->do($sql, [ $DoPaymentOnBasket, $PaymentTypeAlias ]);

    return;
}
1;
