#========================================================================================
# package      DE_EPAGES::Payment::API::TLE::PaymentHandler
# state        public
#----------------------------------------------------------------------------------------
# description  Implements TLE functions for payments.
#========================================================================================
package DE_EPAGES::Payment::API::TLE::PaymentHandler;
use base DE_EPAGES::TLE::API::LoopHandler;

use strict;
use DE_EPAGES::Core::API::Log qw ( GetLog );

#========================================================================================
# function     RegisterHandlerProc
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RegisterHandlerProc( $hParams );
#----------------------------------------------------------------------------------------
# description  this function registers this attribute handle on the processor via hook
#               function on hook TLEProcessorRegistration
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments keys :
#               <ul>
#                   <li>Processor - tle processor - object
#               </ul> | ref.hash
#========================================================================================
sub RegisterHandlerProc {
    my ($Params) = @_;
    GetLog->debug( 'PaymentHandler.RegisterHandlerProc' );

    __PACKAGE__->new()->register( $Params->{'Processor'} );
    return;
}

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler = new DE_EPAGES::Payment::API::TLE::PaymentHandler;
# example      my $PaymentHandler  = new DE_EPAGES::Payment::API::TLE::PaymentHandler;
#----------------------------------------------------------------------------------------
# description  create a new handler
#----------------------------------------------------------------------------------------
# return       $Handler | blessed object | object
#========================================================================================
sub new { bless {}, shift; }

#========================================================================================
# function     register
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->register( $Processor );
#----------------------------------------------------------------------------------------
# description  this function register this handle on the processor
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
#========================================================================================
sub register {
    my $self = shift;
    my ($Processor) = @_;
    GetLog->debug( 'PaymentHandler.register' );

    foreach (qw( PRICELIMIT DISCOUNT CanUsePayment )) {
        $Processor->registerHandler('FunctionHandler', $self, $_);
    }

    return;
}

#========================================================================================
# function     PRICELIMIT
# state        public
#----------------------------------------------------------------------------------------
# syntax       #PRICELIMIT(#Type, #PaymentMethod, #Currency)
# example      #LOCAL("CurrencyID", #INPUT.Currency)
#                 #LOCAL("ListPrice", #PRICELIMIT(#Type, #PaymentMethod))
#                   #IF(#DEFINED(#ListPrice))
#                     #ListPrice[money]
#                   #ENDIF
#                 #ENDLOCAL
#               #ENDLOCAL
#----------------------------------------------------------------------------------------
# description  Returns the price limit of a payment method for a given currency and tax model.
#----------------------------------------------------------------------------------------
# input        #Processor | tle processor | object
# input        #Type | type of price value   MinPriceValue or MaxPriceValue | string
# input        #PaymentMethod | product object | object
# input        #CurrencyID | currency code, e.g. "EUR" | string
#========================================================================================
sub PRICELIMIT {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($Type, $PaymentMethod, $CurrencyID) = @$aParams;

    return $PaymentMethod->getMaxPriceValue($CurrencyID) if $Type eq 'MaxPriceValue';
    return $PaymentMethod->getMinPriceValue($CurrencyID) if $Type eq 'MinPriceValue';

    return ;

}

#========================================================================================
# function     DISCOUNT
# state        public
#----------------------------------------------------------------------------------------
# syntax       #DISCOUNT(#PaymentMethod, #Currency)
# example      #LOCAL("CurrencyID", #INPUT.Currency)
#                 #LOCAL("Discount", #DISCOUNT(#PaymentMethod,#CurrencyID))
#                   #IF(#DEFINED(#Discount))
#                     #Discount[money]
#                   #ENDIF
#                 #ENDLOCAL
#               #ENDLOCAL
#----------------------------------------------------------------------------------------
# description  Returns the discount or fee of a payment method for a given currency.
#----------------------------------------------------------------------------------------
# input        #Processor | tle processor | object
# input        #PaymentMethod | product object | object
# input        #CurrencyID | currency code, e.g. "EUR" | string
#========================================================================================
sub DISCOUNT {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($PaymentMethod, $CurrencyID) = @$aParams;

    return $PaymentMethod->getPaymentMethodDiscount($CurrencyID);


}

#========================================================================================
# function     CanUsePayment
# state        public
#----------------------------------------------------------------------------------------
# syntax       #CanUsePayment(#PaymentMethod, #Container)
# example      #LOCAL("UsePayment", #CanUsePayment(#PaymentMethod.Object, #LineItemContainer.Object))
#----------------------------------------------------------------------------------------
# description  Calls canAddToBasket of #PaymentMethod to determine if #PaymentMethod
#               can be added to #Container.
#----------------------------------------------------------------------------------------
# input        #Processor | tle processor | object
# input        #PaymentMethod | payment method object | object
# input        #Container | container object, e.g. #LineItemContainer | object
# return       #Result | true if #PaymentMethod can be added to #Container | boolean
#========================================================================================
sub CanUsePayment {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($PaymentMethod, $Container) = @$aParams;

    return unless $PaymentMethod && $Container;

    my $hOptions = $Container->get([qw(CurrencyID TaxModel LocaleID)]);

    # add LineItemContainer to hOptions to enable additional checks in canAddToBasket.
    $hOptions->{LineItemContainer} = $Container;

    return $PaymentMethod->canAddToBasket($hOptions);
}

1;
