#========================================================================================
# package      DE_EPAGES::Payment::t::unit::PaymentGroup
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for API::Object::PaymentGroup.pm
#========================================================================================
package DE_EPAGES::Payment::t::unit::PaymentGroup;
use base DE_EPAGES::Test::API::UnitTest::Test;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Test::Mock::Mockify;
use Test::More;
use Test::Exception;
use strict;

sub test {
    my $self = shift;

    $self->test_isUsable_ErrorCase();
    $self->test_isUsable_WithoutPaymentTypes();
    $self->test_isUsable_MultiplePaymentTypes_withoutFeature();
    $self->test_isUsable_MultiplePaymentTypes_NoMatchingGroup();
    $self->test_isUsable_MultiplePaymentTypes_WithoutPaymentGroups();
    $self->test_isUsable_MultiplePaymentTypes_withOneFeaturedPaymentType();

    $self->test_getPaymentTypes_ErrorCase();
    $self->test_getPaymentTypes_WithoutPaymentTypes();
    $self->test_getPaymentTypes_MultiplePaymentTypes_withoutFeature();
    $self->test_getPaymentTypes_MultiplePaymentTypes_NoMatchingGroup();
    $self->test_getPaymentTypes_MultiplePaymentTypes_WithoutPaymentGroups();
    $self->test_getPaymentTypes_MultiplePaymentTypes_withOneFeaturedPaymentType();

    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_isUsable_ErrorCase {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createPaymentGroup_SUTwithError();
    my $Result;

    lives_ok( sub { $Result = $SUT->isUsable(); }, "$SubTestName - Tests if the isUsable is not dying in case of an error." );

    is($Result, 0,"$SubTestName - return 0 if any Error appears.");
}
#----------------------------------------------------------------------------------------
sub test_isUsable_WithoutPaymentTypes {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createPaymentGroup_SUT({
      'PaymentGroupName' => 'PaymentGroup_UnitTest',
      'PaymentTypesInShop' => [],
    } );
    my $Shop = $self->_createShop();

    my $Result = $SUT->isUsable($Shop);

    is($Result, 0, "$SubTestName - Expect false since there is no payment type on shop.");
}
#----------------------------------------------------------------------------------------
sub test_isUsable_MultiplePaymentTypes_withoutFeature {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createPaymentGroup_SUT({
      'PaymentGroupName' => 'PaymentGroup_UnitTest',
      'PaymentTypesInShop' => [
          $self->_createPaymentType({'Alias' => 'NotFeaturedPaymentType', 'PaymentGroupName' =>'PaymentGroup_UnitTest', 'isUsable' => 0}),
          $self->_createPaymentType({'Alias' => 'NotFeaturedPaymentType', 'PaymentGroupName' =>'PaymentGroup_UnitTest', 'isUsable' => 0}),
      ],
    } );
    my $Shop = $self->_createShop();

    my $Result = $SUT->isUsable($Shop);

    is($Result, 0, "$SubTestName - Expect false since the payment group have not a single featured payment type.");
}
#----------------------------------------------------------------------------------------
sub test_isUsable_MultiplePaymentTypes_NoMatchingGroup {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createPaymentGroup_SUT({
      'PaymentGroupName' => 'PaymentGroup_UnitTest',
      'PaymentTypesInShop' => [
          $self->_createPaymentType({'Alias' => 'FeaturedPaymentType', 'PaymentGroupName' =>'PaymentGroup_Other'}),
          $self->_createPaymentType({'Alias' => 'NotFeaturedPaymentType', 'PaymentGroupName' =>'PaymentGroup_Other'}),
      ],
    } );
    my $Shop = $self->_createShop();

    my $Result = $SUT->isUsable($Shop);

    is($Result, 0, "$SubTestName - Expect false since there is no payment type which match the payment group.");
}
#----------------------------------------------------------------------------------------
sub test_isUsable_MultiplePaymentTypes_WithoutPaymentGroups {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createPaymentGroup_SUT({
      'PaymentGroupName' => 'PaymentGroup_UnitTest',
      'PaymentTypesInShop' => [
          $self->_createPaymentType({'Alias' => 'FeaturedPaymentType'}),
          $self->_createPaymentType({'Alias' => 'NotFeaturedPaymentType'}),
      ],
    } );
    my $Shop = $self->_createShop();

    my $Result = $SUT->isUsable($Shop);

    is($Result, 0, "$SubTestName - Expect false since the payment types don't have payment groups.");
}
#----------------------------------------------------------------------------------------
sub test_isUsable_MultiplePaymentTypes_withOneFeaturedPaymentType {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createPaymentGroup_SUT({
      'PaymentGroupName' => 'PaymentGroup_UnitTest',
      'PaymentTypesInShop' => [
          $self->_createPaymentType({'Alias' => 'NotFeaturedPaymentType', 'PaymentGroupName' =>'PaymentGroup_UnitTest', 'isUsable' => 0}),
          $self->_createPaymentType({'Alias' => 'FeaturedPaymentType', 'PaymentGroupName' =>'PaymentGroup_UnitTest', 'isUsable' => 1}),
      ],
    } );
    my $Shop = $self->_createShop();

    my $Result = $SUT->isUsable($Shop);

    ok($Result, "$SubTestName - Expect true if at least one payment typ is featured.");
}
#----------------------------------------------------------------------------------------
sub test_getPaymentTypes_ErrorCase {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createPaymentGroup_SUTwithError();

    throws_ok( sub { $SUT->getPaymentTypes() },
                   qr/TestErrorInPaymentGroup/,
                   "$SubTestName - Tests if an error will be re-thrown since we can't handle it."
    );

}
#----------------------------------------------------------------------------------------
sub test_getPaymentTypes_WithoutPaymentTypes {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createPaymentGroup_SUT({
      'PaymentGroupName' => 'PaymentGroup_UnitTest',
      'PaymentTypesInShop' => [],
    } );
    my $Shop = $self->_createShop();

    my $aPaymentTypes = $SUT->getPaymentTypes($Shop);

    is_deeply($aPaymentTypes, [],"$SubTestName - test if paymenttypes returns empty array");
}
#----------------------------------------------------------------------------------------
sub test_getPaymentTypes_MultiplePaymentTypes_withoutFeature {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createPaymentGroup_SUT({
      'PaymentGroupName' => 'PaymentGroup_UnitTest',
      'PaymentTypesInShop' => [
          $self->_createPaymentType({'Alias' => 'NotFeaturedPaymentType', 'PaymentGroupName' =>'PaymentGroup_UnitTest', 'isUsable' => 0}),
          $self->_createPaymentType({'Alias' => 'NotFeaturedPaymentType', 'PaymentGroupName' =>'PaymentGroup_UnitTest', 'isUsable' => 0}),
      ],
    } );
    my $Shop = $self->_createShop();

    my $aPaymentTypes = $SUT->getPaymentTypes($Shop);

    is_deeply($aPaymentTypes, [],"$SubTestName - test if paymenttypes returns empty array");
}
#----------------------------------------------------------------------------------------
sub test_getPaymentTypes_MultiplePaymentTypes_NoMatchingGroup {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createPaymentGroup_SUT({
      'PaymentGroupName' => 'PaymentGroup_UnitTest',
      'PaymentTypesInShop' => [
          $self->_createPaymentType({'Alias' => 'FeaturedPaymentType', 'PaymentGroupName' =>'PaymentGroup_Other'}),
          $self->_createPaymentType({'Alias' => 'NotFeaturedPaymentType', 'PaymentGroupName' =>'PaymentGroup_Other'}),
      ],
    } );
    my $Shop = $self->_createShop();

    my $aPaymentTypes = $SUT->getPaymentTypes($Shop);

    is_deeply($aPaymentTypes, [],"$SubTestName - test if paymenttypes returns empty array");
}
#----------------------------------------------------------------------------------------
sub test_getPaymentTypes_MultiplePaymentTypes_WithoutPaymentGroups {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createPaymentGroup_SUT({
      'PaymentGroupName' => 'PaymentGroup_UnitTest',
      'PaymentTypesInShop' => [
          $self->_createPaymentType({'Alias' => 'FeaturedPaymentType'}),
          $self->_createPaymentType({'Alias' => 'NotFeaturedPaymentType'}),
      ],
    } );
    my $Shop = $self->_createShop();

    my $aPaymentTypes = $SUT->getPaymentTypes($Shop);

    is_deeply($aPaymentTypes, [], "$SubTestName - Expect false since the payment types don't have payment groups.");
}
#----------------------------------------------------------------------------------------
sub test_getPaymentTypes_MultiplePaymentTypes_withOneFeaturedPaymentType {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createPaymentGroup_SUT({
      'PaymentGroupName' => 'PaymentGroup_UnitTest',
      'PaymentTypesInShop' => [
          $self->_createPaymentType({'Alias' => 'NotFeaturedPaymentType', 'PaymentGroupName' =>'PaymentGroup_UnitTest', 'isUsable' => 0}),
          $self->_createPaymentType({'Alias' => 'FeaturedPaymentType', 'PaymentGroupName' =>'PaymentGroup_UnitTest', 'isUsable' => 1}),
      ],
    } );
    my $Shop = $self->_createShop();

    my $aPaymentTypes = $SUT->getPaymentTypes($Shop);
    my $PaymentTypesLength = scalar @{ $aPaymentTypes };
    my $PaymentType1 = $aPaymentTypes->[0];

    is($PaymentTypesLength, 1,"$SubTestName - Check if paymenttypes returns 1 element");
    is($PaymentType1->alias(), 'FeaturedPaymentType',"$SubTestName - Check if paymenttypes returns correct paymenttype");
}
#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _createShop {
    my $self = shift;
    my ($Arguments) = @_;

    my $Mockify = $self->mockFactory()->buildShop({
            'Alias' => 'TestShop',
        });

    $Mockify->addMockWithConditionalReturn('featureMaxValue', 1, [{'string' => 'FeaturedPaymentType'}]);
    $Mockify->addMockWithConditionalReturn('featureMaxValue', 0, [{'string' => 'NotFeaturedPaymentType'}]);

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createPaymentType {
    my $self = shift;
    my ($hParams) = @_;

    my $PaymentGroup = $self->_createPaymentGroup( $hParams->{'PaymentGroupName'} );
    my $hAttributes = {
        'Alias' => $hParams->{'Alias'},
        'PaymentGroup' => $PaymentGroup,
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Payment::API::Object::PaymentType',
        $hAttributes
    );
    $Mockify->addMockWithReturnValueAndParameterCheck('isUsable', $hParams->{'isUsable'}, [{'object'=>'DE_EPAGES::Shop::API::Object::Shop'}]);

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createPaymentGroup {
    my $self = shift;
    my ($Alias) = @_;

    return unless defined $Alias;
    my $hAttributes = {
        'Alias' => $Alias,
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Payment::API::Object::PaymentGroup',
        $hAttributes
    );

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createPaymentTypesFolder {
    my $self = shift;
    my ($hParams) = @_;

    my $hAttributes = {
        'Children' => $hParams->{'PaymentTypesInShop'},
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Object::API::Object::Object',
        $hAttributes
    );

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createPaymentGroup_SUTwithError {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Payment::API::Object::PaymentGroup',
        {}
    );
    $Mockify->mock('_LoadObjectByPath', sub{
        Error('TestErrorInPaymentGroup');
    });

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createPaymentGroup_SUT {
    my $self = shift;
    my ($hParams) = @_;

    my $hAttributes = {
        'Alias' => $hParams->{'PaymentGroupName'}
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
                   'DE_EPAGES::Payment::API::Object::PaymentGroup',
                   $hAttributes
              );
    my $PaymentTypesFolder = $self->_createPaymentTypesFolder($hParams);
    $Mockify->mock('_LoadObjectByPath', $PaymentTypesFolder , [{'string'=>'/PaymentTypes'}]);

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();