#========================================================================================
# package      DE_EPAGES::Shop::API::ShopUtil
# state        public
#----------------------------------------------------------------------------------------
# description  Utility functions which operate on the Shop object
#========================================================================================
package DE_EPAGES::Shop::API::ShopUtil;

use strict;
use List::MoreUtils qw(uniq);
#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::Shop::API::ShopUtil
#========================================================================================
sub new {
    my $class = shift;
    my ($Shop) = @_;
    my $self  = bless {'Shop' => $Shop}, $class;
    return $self;
}

#========================================================================================
# function     addMandatoryAddressFields
# state        public
#----------------------------------------------------------------------------------------
# syntax       addMandatoryAddressFields( $aBillingAddressFields, $aShippingAddressFields );
#----------------------------------------------------------------------------------------
# description  Makes certain address fields mandatory.
#----------------------------------------------------------------------------------------
# input        $aBillingAddressFields | Comma-separated list of billing  
#                                         address fields which must be mandatory. | string
# input        $aShippingAddressFields | Comma-separated list of shipping  
#                                         address fields which must be mandatory. | string
#========================================================================================
sub addMandatoryAddressFields {
    my $self = shift;
    my ($aBillingAddressFields, $aShippingAddressFields) = @_;

    $self->addMandatoryBillingAddressFields($aBillingAddressFields);
    $self->addMandatoryShippingAddressFields($aShippingAddressFields);

    return;
}

sub addMandatoryBillingAddressFields {
    my $self = shift;
    my ($aBillingAddressFields) = @_;

    $self->_addFields($aBillingAddressFields, 'BillingAddressSetMandatoryFields');
    $self->_addFields($aBillingAddressFields, 'BillingAddressSetVisibleFields');

    return;
}

sub addMandatoryShippingAddressFields {
    my $self = shift;
    my ($aShippingAddressFields) = @_;

    $self->_addFields($aShippingAddressFields, 'ShippingAddressSetMandatoryFields');
    $self->_addFields($aShippingAddressFields, 'ShippingAddressSetVisibleFields');

    return;
}

sub _addFields {
    my $self = shift;
    my ($aFields, $FieldName) = @_;

    return unless defined $aFields;

    my $Shop = $self->{'Shop'};

    my @aCurrentFields = split(/,/ ,$Shop->get($FieldName));
    my @AllFields = uniq(@aCurrentFields, @$aFields);

    my $AllFieldsString = join(',', @AllFields);

    $Shop->set({
        $FieldName => $AllFieldsString
    });

    return;
}

#========================================================================================
# function     maxFeatureValuePaymentMethodsReached
# state        public
#----------------------------------------------------------------------------------------
# syntax       maxFeatureValuePaymentMethodsReached( );
#----------------------------------------------------------------------------------------
# description  Check if the maxFeatureValue for payment methods is reached.
#----------------------------------------------------------------------------------------
# return       true if it is possible to create another payment method  | boolean
#========================================================================================
sub maxFeatureValuePaymentMethodsReached {
    my $self = shift;

    my $Shop = $self->{'Shop'};
    my $MaximalAmountPaymentMethods = $Shop->featureMaxValue('PaymentMethods');
    my $CurrentPaymentMethodsInShop = $Shop->featureCurrentValue('PaymentMethods');

    if( $CurrentPaymentMethodsInShop >= $MaximalAmountPaymentMethods ){
      return 1;
    }else {
      return 0;
    }
}
1;