use strict;
use Getopt::Long;
use feature 'switch';
use Test::More;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::File qw( ExistsDirectory GetFileNames );
use DE_EPAGES::Object::API::XMLStructureImportHandler;
use DE_EPAGES::XML::API::XML::CartridgeInstallerImportHandler qw( GetCartridges );

sub Main {
    my $Help;
    my $CompanyName;
    my $Cartridge;

    GetOptions(
        'help'             => \$Help,
        'company=s'        => \$CompanyName,
        'cartridge=s'      => \$Cartridge,
    );

    usage() if $Help;

    $CompanyName = 'DE_EPAGES' unless( defined $CompanyName);

    #files that dont have to be checked
    my $hExceptions = {
        'EbayRegions.xml' => 1,
        'Languages.xml' => 1,
        'DistributorShopTypes.xml' => 1,
        'ShopTypes.xml' => 1,
        'DefaultShop.xml' => 1,
        'FlexShopType.xml' => 1,
    };

    my $hStatistic;

    #check Cartridge xml files for each store type
    my $hStoreTypeCartridges = GetCartridges();
    my $BaseDir = $ENV{'EPAGES_CARTRIDGES'}.'/'.$CompanyName;

    foreach my $StoreType (keys %$hStoreTypeCartridges ){
        $hStatistic = {};
        my $aCartridgeDirs = [
            map {
                $BaseDir .'/'. (split(/::/, $_))[1]
            } @{ $hStoreTypeCartridges->{$StoreType} }
        ];

        my %CartridgeDirs; # check for duplicate cartridges

        #check every cartridge
        foreach my $CartridgeDir (@$aCartridgeDirs){

            #ignore exceptions
            next if $CartridgeDir =~ /\/CVS$/;
            next if ((defined $Cartridge) and not ($CartridgeDir =~ /$Cartridge/));

            # check for duplicates
            if( exists $CartridgeDirs{$CartridgeDir} ) {
                die "duplicate cartridge dir: $CartridgeDir in store type $StoreType. See Cartridges.xml\n";
            }
            $CartridgeDirs{$CartridgeDir} = 1;

            if (ExistsDirectory($CartridgeDir . '/Database/XML')){

                #get *.xml files under ../Database/XML
                my $aXMLFiles = GetFileNames($CartridgeDir . '/Database/XML', {'recursive' => 1} );

                #check every file
                foreach my $XMLFile (@$aXMLFiles){

                    #get file name from path
                    my $FileName = $XMLFile;
                    $FileName =~ s{
                        .*\/(
                        [\_0-9a-zA-Z\.]+    #file name
                        \.xml               #file type
                        )$
                    }{$1}x;

                    #ignore exceptions
                    next unless $XMLFile =~ /.+\.xml$/;
                    next if $XMLFile =~ /\/SQL\//;
                    next if $XMLFile =~ /\/mysql\//;
                    next if $hExceptions->{$FileName};
                    next if $XMLFile =~ m{ShopChildDependencies\.xml$}x;
                    next if $XMLFile =~ /pt-BR\.xml$/;
                    next if $XMLFile =~ /en-US\.xml$/;

                    if ($XMLFile =~ /Translation\..*\.(\w\w|\w\w-\w\w)\.xml$/x){
                        #make check for translation files
                        $hStatistic->{'CheckedFiles'}->{$XMLFile} = 1;
                        CheckTranslationFile($XMLFile,$hStatistic);
                    }
                    else {
                        #make check for normal xml file
                        $hStatistic->{'CheckedFiles'}->{$XMLFile} = 1;
                        CheckXMLFile($XMLFile,$hStatistic);
                    }
                }
            }
        }

        #test the builded statistic
        TestHelptopics($hStatistic);
   }

    ok(1, 'Tests run');
}

#========================================================================================
# function     CheckXMLFile
#----------------------------------------------------------------------------------------
# syntax       CheckXMLFile($XMLFile,$hStatistic);
#----------------------------------------------------------------------------------------
# description  Runs tests for a normal xml file.
#----------------------------------------------------------------------------------------
# input        $FilePath | path to xml file | string
# input        $hStatistic  | will filled with check Informations | ref.hash
#========================================================================================
sub CheckXMLFile {
    my ($FilePath, $hStatistic) = @_;

    #define exception tags
    my $hExceptions = {
        '' => 1,
        'epages' => 1,
        'Dependency' => 1,
    };

    my $hTemporarySkipClass = {
        'Dependency' => 1,
        'ShippingLevel' => 1,
        'AttributeValue' => 1,
    };

    #generate xml structure from file
    my $XMLStructImporter = DE_EPAGES::Object::API::XMLStructureImportHandler->new();
    my $XMLStruct = $XMLStructImporter->getXMLStructure($FilePath, {'distinguish_multiple_appearence' => 1});

    #complete expected delete-Attributes
    AddDeleteIfParentHasDelete($XMLStruct, {'File' => $FilePath} );

    #check every element in xml structure
    foreach my $Element (@{$XMLStruct->getElements()}){

        #ignore exceptions
        next if $hExceptions->{$Element};

        #get class of element in xml structure
        my $Class = $XMLStruct->getXMLStructAttribute($Element, 'Class');

        #special AttributeValue handling
        if ($Class eq 'AttributeValue'){

            #get language code
            my $LangCode = $XMLStruct->getIDValue($Element, 'Language');

            #get cleaned tag path
            my $TagPath = _GetNameFreeFromAppearence($Element);

            #check if element is defined in an other file
            if ( exists $hStatistic->{'TranslationXMLTagReference'}->{$LangCode}->{$TagPath} ){
                #element already referenced in an other file
                ok(0,
                    "multiple reference of element '$Element' in file '$FilePath' and "
                    . $hStatistic->{'TranslationXMLTagReference'}->{$LangCode}->{$TagPath}
                );
            }

            #set file where element is defined
            $hStatistic->{'TranslationXMLTagReference'}->{$LangCode}->{$TagPath} = $FilePath;
        }

        #next if class should be skipped
        next if $hTemporarySkipClass->{$Class};

        #fill visited classes to statistic
        $hStatistic->{'CheckedClasses'}->{$Class} = 1;
        $hStatistic->{'CheckedTagNames'}->{$XMLStruct->getXMLStructAttribute($Element, 'TagName')} = 1;

        #check if element has an delete or a reference attribute
        if (
            not(
                $XMLStruct->existsTagAttribute($Element, 'reference') or
                $XMLStruct->existsTagAttribute($Element, 'delete')
            )
        ){
            my $IsExceptionFolderForRefDel = IsExceptionFolderRefDel($XMLStruct, $Element);

            #check if delete or reference have to exists or there is an exception folder
            unless($IsExceptionFolderForRefDel){
                ok(0, "file '$FilePath' element '$Element' delete or reference does not exists");
            }
        }
        elsif($XMLStruct->existsTagAttribute($Element, 'delete')){

            #get cleaned tag path
            my $TagPath = _GetNameFreeFromAppearence($Element);

            #check if element is defined in an other file
            if ( exists $hStatistic->{'XMLTagDefinition'}->{$TagPath} ){
                #element already defined in an other file
                if ($FilePath ne $hStatistic->{'XMLTagDefinition'}->{$TagPath} ){
                    #different file create failed test message
                    ok(0, "multiple definition of element '$Element' in file '$FilePath' and " . $hStatistic->{'XMLTagDefinition'}->{$TagPath});
                }
                elsif ($TagPath ne $Element) {
                    #multiple appearance
                    ok(0, "multiple definition of element '$Element' in same file '$FilePath'");
                }


            }

            #set file where element is defined
            $hStatistic->{'XMLTagDefinition'}->{$TagPath} = $FilePath;
        }

        #check for additional statistic
        given($Element){
            when (IsHelpTopic($XMLStruct, $_)){#element is a helptopic
                RegisterHelpTopic(
                    _GetNameFreeFromAppearence($Element),
                    $hStatistic,
                    $FilePath,
                    $XMLStruct
                );
            }
        }
    }
}

#========================================================================================
# function     CheckTranslationFile
#----------------------------------------------------------------------------------------
# syntax       CheckTranslationFile($XMLFile,$hStatistic);
#----------------------------------------------------------------------------------------
# description  Runs tests for a translation xml file.
#----------------------------------------------------------------------------------------
# input        $FilePath | path to xml file | string
# input        $hStatistic  | will filled with check Informations | ref.hash
#========================================================================================
sub CheckTranslationFile{
    my ($FilePath, $hStatistic) = @_;

    #define exception tags
    my $hExceptions = {
        '' => 1,
        'epages' => 1,
    };

    #generate xml structure from file
    my $XMLStructImporter = DE_EPAGES::Object::API::XMLStructureImportHandler->new();
    my $XMLStruct = $XMLStructImporter->getXMLStructure($FilePath, {'distinguish_multiple_appearence' => 1});

    #check every element in xml structure
    foreach my $Element (@{$XMLStruct->getElements()}){

        #ignore exceptions
        next if $hExceptions->{$Element};

        #get class of element in xml structure
        my $Class = $XMLStruct->getXMLStructAttribute($Element, 'Class');

        #object tags can be referenced in 'no-trans' translation files and in translation files
        next if (($Class eq 'Object') and ($FilePath =~ /\.Notrans\./));

        given($Element){
            when (IsHelpTopic($XMLStruct,$_)){#element is a helptopic
                RegisterHelpTopic(
                    _GetNameFreeFromAppearence($Element),
                    $hStatistic,
                    $FilePath,
                    $XMLStruct
                );
            }
            when (IsHelpTopicAttribute($_)){#element is a helptopic attribute
                RegisterHelpTopicAttribute(
                    _GetNameFreeFromAppearence($Element),
                    $hStatistic,
                    $FilePath,
                    $XMLStruct
                );
            }
            default{
                #ignore <epages> and <Language> tags
                next if(($Element eq 'epages') or ($Element eq 'Language'));

                #get language code
                my $LangCode = $XMLStruct->getIDValue($Element, 'Language');

                #if langcode is not defined get it from file name (e.g. object tags have no Language value)
                unless($LangCode){
                    $LangCode = $FilePath;
                    $LangCode =~ s/.*\.(\w\w)\.xml$/$1/x;
                }

                #get cleaned tag path
                my $TagPath = _GetNameFreeFromAppearence($Element);

                #check if element is defined in an other file
                if ( exists $hStatistic->{'TranslationXMLTagReference'}->{$LangCode}->{$TagPath} ){
                    #element already referenced in an other file
                    ok(0,
                        "multiple reference of element '$Element' in file '$FilePath' and "
                        . $hStatistic->{'TranslationXMLTagReference'}->{$LangCode}->{$TagPath}
                    );
                }

                #set file where element is defined
                $hStatistic->{'TranslationXMLTagReference'}->{$LangCode}->{$TagPath} = $FilePath;
            }
        }
    }
}

#========================================================================================
# function     AddDeleteIfParentHasDelete
#----------------------------------------------------------------------------------------
# syntax       AddDeleteIfParentHasDelete($XMLStruct, $hInfo);
#----------------------------------------------------------------------------------------
# description  Adds delete attribute for every element in xml structure that have a parent with delete attribute.
#----------------------------------------------------------------------------------------
# input        $XMLStruct | xml structure object | object
# input        $hInfo  | contains information for recursive calling | ref.hash
#========================================================================================
sub AddDeleteIfParentHasDelete {
    my ($XMLStruct, $hInfo) = @_;

    #for every element in xml structure
    foreach my $Element (@{$XMLStruct->getElements()}){
        #ignore element that already have a delete attribute
        if ($XMLStruct->existsTagAttribute($Element, 'delete')){
            #if (ParentWithDeleteExists($XMLStruct, $Element)){#parent have a delete attribute and subobject have a redundant delete
            #    ok(
            #        0,
            #        "file '" . $hInfo->{'File'} . "' referenced element '$Element' has a redundant delete attribute in tag"
            #    );
            #}
            next;
        };

        if (
            $XMLStruct->getTagAttribute($Element, 'reference') and
            IsFirstAppearence($Element)
        ){#first tag appearence and referenced
            if (ParentWithDeleteExists($XMLStruct, $Element)){#parent have a delete attribute
                ok(
                    0,
                    'file \'' . $hInfo->{'File'} . "' referenced element '$Element' has no parent tag with delete"
                );
            }
        }
        elsif (not $XMLStruct->existsTagAttribute($Element, 'delete')){
            #call recursive function
            ParentWithDeleteExists($XMLStruct,$Element,{'setdeletes' => 1});
        }
    }
}

#========================================================================================
# function     IsFirstAppearence
#----------------------------------------------------------------------------------------
# syntax       $IsFirst = IsFirstAppearence($Element);
#----------------------------------------------------------------------------------------
# description  Check element name if no appearence string is contained.
#----------------------------------------------------------------------------------------
# input        $Element | element of xml structure object | string
# return       $IsFirst | true if this is the first appearence of the object | boolean
#========================================================================================
sub IsFirstAppearence {
    my ($Element) = @_;

    return not $Element =~ m{
        \:appearance  #signal of multiple appearance of tag
        \(\d+\)$            #appearance counter at the ending of element string
    }x;
}

#========================================================================================
# function     IsHelpTopic
#----------------------------------------------------------------------------------------
# syntax       IsHelpTopic($Element);
#----------------------------------------------------------------------------------------
# description  Check element name if belong to a HelpTopic.
#----------------------------------------------------------------------------------------
# input        $XMLStructure | xml structure object | object
# input        $Element | element of xml structure object | string
# return       $IsHelpTopic | element is a help topic | boolean
#========================================================================================
sub IsHelpTopic{
    my ($XMLStruct, $Element) = @_;

    return 1 if ($XMLStruct->getXMLStructAttribute($Element,'TagName') eq 'HelpTopic');
    return 0;
}

#========================================================================================
# function     IsHelpTopicAttribute
#----------------------------------------------------------------------------------------
# syntax       IsHelpTopicAttribute($Element);
#----------------------------------------------------------------------------------------
# description  Check element name if belong to a HelpTopic attribute.
#----------------------------------------------------------------------------------------
# input        $Element | element of xml structure object | string
# return       $IsHelpTopicAttribute | element is a help topic attribute | boolean
#========================================================================================
sub IsHelpTopicAttribute{
    my ($Element) = @_;

    ## no critic qw( ProhibitComplexRegexes )
    return $Element =~ m{
        \/HelpTopics\/         #Search Text on beginning
        [a-zA-Z\_\-0-9\.]+      #Helptopic name
        \/AttributeValue\{      # ../AttributeValue{
        Language\-\>.+\;        #Language->...;
        Name\-\>.+\}$           #Name->...} at the end
    }x;
    ## use critc
}

#========================================================================================
# function     _GetNameFreeFromAppearence
#----------------------------------------------------------------------------------------
# syntax       _GetNameFreeFromAppearence($Element);
#----------------------------------------------------------------------------------------
# description  Returns element name free from appearence strings.
#----------------------------------------------------------------------------------------
# input        $Element | element of xml structure object | string
# return       $CleanElement | element of xml structure object free from appearence strings| string
#========================================================================================
sub _GetNameFreeFromAppearence{
    my ($Element) = @_;

    $Element =~ s{
        \:appearance    #apearence signal
        \(\d+\)         #appearance counter
    }{}xg;

    return $Element;
}

#========================================================================================
# function     ParentWithDeleteExists
#----------------------------------------------------------------------------------------
# syntax       ParentWithDeleteExists($XMLStruct, $Element, $hOptions);
#----------------------------------------------------------------------------------------
# description  Returns true if parent element with delete attribute exists.
#----------------------------------------------------------------------------------------
# input        $XMLStruct | xml structure object | object
# input        $Element | element of xml structure object | string
# input        $hOptions  | contains information for recursive calling | ref.hash
# return       $ParentExists | true if parent with delete attribute exists | bool
#========================================================================================
sub ParentWithDeleteExists {
    my ($XMLStruct, $Element, $hOptions) = @_;

    #no parent with delete exists
    return 0 if $Element eq '';

    #get parent element
    my $Parent = $XMLStruct->parentElement($Element);

    if ($XMLStruct->existsTagAttribute($Parent, 'delete')){#direkt parent has delete attribute
        $XMLStruct->setTagAttribute($Element, 'delete', 1) if $hOptions->{'setdeletes'};
        return 1;
    }
    else{#direkt parent has no delete attribute, have to go on recursive
        return ParentWithDeleteExists($XMLStruct, $Parent, $hOptions);
    }
}

#========================================================================================
# function     RegisterHelpTopic
#----------------------------------------------------------------------------------------
# syntax       RegisterHelpTopic($Element, $hStatistic, $FilePath, $XMLStruct);
#----------------------------------------------------------------------------------------
# description  Register HelpTopic in statistic.
#----------------------------------------------------------------------------------------
# input        $Element | element of xml structure object | string
# input        $hStatistic  | will filled with check Informations | ref.hash
# input        $FilePath | path to xml file | string
# input        $XMLStruct | xml structure object | object
#========================================================================================
sub RegisterHelpTopic{
    my ($Element, $hStatistic, $FilePath, $XMLStruct) = @_;

    if ($XMLStruct->existsTagAttribute($Element, 'delete')){
        $hStatistic->{'HelpTopics'}->{$Element}->{'DefinedAt'}->{$FilePath} = 1;
    }
    elsif ($XMLStruct->existsTagAttribute($Element, 'reference')){
        $hStatistic->{'HelpTopics'}->{$Element}->{'ReferencedAt'}->{$FilePath} = 1;
    }
    else{
        $hStatistic->{'HelpTopics'}->{$Element}->{'ReferencedAt'}->{$FilePath} = 1;
    }
}

#========================================================================================
# function     RegisterHelpTopicAttribute
#----------------------------------------------------------------------------------------
# syntax       RegisterHelpTopicAttribute($Element, $hStatistic, $FilePath, $XMLStruct);
#----------------------------------------------------------------------------------------
# description  Register HelpTopic attribute in statistic.
#----------------------------------------------------------------------------------------
# input        $Element | element of xml structure object | string
# input        $hStatistic  | will filled with check Informations | ref.hash
# input        $FilePath | path to xml file | string
# input        $XMLStruct | xml structure object | object
#========================================================================================
sub RegisterHelpTopicAttribute{
    my ($Element, $hStatistic, $FilePath, $XMLStruct) = @_;

    my $HelpTopic = $XMLStruct->parentElement($Element);

    $hStatistic->{'HelpTopics'}->{$HelpTopic}->{'AttrDefinedAt'}->{$FilePath} = 1;
}

#========================================================================================
# function     TestHelptopics
#----------------------------------------------------------------------------------------
# syntax       TestHelptopics($hStatistic);
#----------------------------------------------------------------------------------------
# description  Test for HelpTopics in statistic.
#----------------------------------------------------------------------------------------
# input        $hStatistic  | will filled with check Informations | ref.hash
#========================================================================================
sub TestHelptopics{
    my ($hStatistic) = @_;

    foreach my $HelpTopic (keys %{$hStatistic->{'HelpTopics'}}){
        if (scalar (keys %{$hStatistic->{'HelpTopics'}->{$HelpTopic}->{'AttrDefinedAt'}}) < 1){
            ok(0, "HelpTopic $HelpTopic has no translation attributes");
        }
    }
}

#========================================================================================
# function     IsExceptionFolderRefDel
#----------------------------------------------------------------------------------------
# syntax       IsExceptionFolderRefDel($XMLStruct,$Element);
#----------------------------------------------------------------------------------------
# description  check if element need to have at least one reference='1' or delete='1'
#               exceptions are folders with following alias 'Actions', 'Attributes', 'SearchDefinitions'
#----------------------------------------------------------------------------------------
# input        $XMLStruct | xml structure | object
# input        $Element  | element in xml structure | string
# return       $IsExceptionFolderRefDel | true if element is an exception folder | bool
#========================================================================================
sub IsExceptionFolderRefDel{
    my ($XMLStruct,$Element) = @_;

    #if no parent exists element is no exception
    return 0 unless ($XMLStruct->existsParentElement($Element));

    #get info
    my $ElementAlias = $XMLStruct->getIDValue($Element, 'Alias');
    my $ParentElement = $XMLStruct->parentElement($Element);
    my $ParentTagName = $XMLStruct->getXMLStructAttribute($ParentElement,'TagName');

    #define exception
    my $hExceptions = {
        'Attributes' => 1,
        'Actions' => 1,
        'SearchDefinitions' => 1,
    };

    #check for exception
    if((defined $hExceptions->{$ElementAlias}) and ($ParentTagName eq 'Class')){
        return 1;
    }

    return 0;
}

sub usage {
    print <<END_USAGE;
Usage:
    This test check for xml notation and content errors.
    perl $0 [options] [flags]

options:
    -company                 name of company folder (DE_EPAGES)
flags:
    -help         show the command line options

Example:
    perl $0 -company ES_EPAGES
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main,
);

done_testing();

1;
